@@ -1,6 +1,6 @@
 #
 #
-# Copyright 2005-2009 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2010 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 #
@@ -24,48 +24,88 @@ our $gui_file_slots = 5;
 # Getopt::GUI::Long safe calling in case it doesn't exist on the system
 #
 sub DTGetOptions {
-
-    # read in the default config file
-    my %config = parseconfig();
-
-    my $nogui = 0;
-    my $have_gui = eval {require Getopt::GUI::Long;};
-
+    my $configfile;
+    my $usegui = 0;
     my $extraopts = [];
 
+    # A special calling case is allowed: --dtconf configfile
+    if ($ARGV[0] eq '--dtconf') {
+	$configfile = $ARGV[1];
+	@ARGV = () if ($#ARGV == 1); # call the GUI or help if only option
+    }
+
+    # read in the default config file
+    my %config = parseconfig($configfile);
+
+    # see if we CAN even do a gui
+    my $have_gui = eval {require Getopt::GUI::Long;};
+
+    # special config; is this used anywhere?
     if ($_[0] eq 'config') {
 	shift @_;
 	$extraopts = shift @_;
     }
 
     # if the default config says not to use a GUI, mark it not to load.
-    if ($config{'usegui'} eq '0' || $config{'usegui'} =~ /^no*/i) {
-	if ($Getopt::GUI::Long::VERSION >= 0.9) {
-	    $nogui = 2;
-	} else {
-	    $nogui = 1;
-	}
+    # (boolconvert defaults nothing=false and we want nothing = true)
+    if (exists($config{'usegui'})) {
+	$usegui = boolconvert($config{'usegui'});
     }
 
+    # allow the environmental override
+    if (exists($ENV{'DT_GUI'})) {
+	$usegui = boolconvert($ENV{'DT_GUI'});
+    }
+
+    # XXX: support --dtconf foo.conf --gui
+
     # then do the right thing based on if we have the Getopt::GUI::Long package
     # and whether we should use it or not by default.
-    if ($nogui != 1 && $have_gui) {
+    if ($have_gui) {
 	require Getopt::Long;
 	import Getopt::GUI::Long;
 	Getopt::GUI::Long::Configure(qw(display_help no_ignore_case));
-	if ($nogui == 2) {
+
+	if (!$usegui &&
+	    $Getopt::GUI::Long::VERSION >= 0.9) {
 	    # we *can perform* a GUI at this point, but the default is off.
 	    # the user can still override using --gui
 	    Getopt::GUI::Long::Configure(qw(no_gui), @$extraopts);
 	}
+
 	return GetOptions(@_);
     }
 
     # fall back to the normal Getopt::Long support
     require Getopt::Long;
     import Getopt::Long;
+
+    # set up for -h / --help output warning
+    my $hashref = 0;
+    my $optionref;
+    if (ref($_[0]) eq 'HASH') {
+	# hash reference options were passed
+	$optionref = $_[0];
+	$hashref = 1;
+	push @_, ["h|help|help-full"];
+    } else {
+	# variable reference options were passed
+	push @_, ["h|help|help-full"], \$optionref;
+    }
+
+    # actually run getoptions
     Getopt::Long::Configure(qw(auto_help no_ignore_case));
-    GetOptions(LocalOptionsMap(@_));
+    my $ret = GetOptions(LocalOptionsMap(@_));
+
+    # check to see if they specified -h or --help
+    if (($optionref && $optionref->{'h'}) ||
+	(!$hashref && $$optionref)) {
+	print
+	  "\nPlease install perl's Getopt::GUI::Long module for help output\n\n";
+	exit 1;
+    }
+
+    return $ret;
 }
 
 sub LocalOptionsMap {
@@ -199,11 +239,11 @@ Net::DNS::SEC::Tools::QWPrimitives - QWizard primitives for DNSSEC-Tools
 
   our @guiargs;
 
-  GetOptions(  ...,
-	       ['GUI:nootherargs',1],
-               ['GUI:otherprimaries',dnssec_tools_get_qwprimitives()],
-  	       ['GUI:submodules','getzonefiles','getzonenames'],
-            );
+  DTGetOptions(  ...,
+	         ['GUI:nootherargs',1],
+                 ['GUI:otherprimaries',dnssec_tools_get_qwprimitives()],
+  	         ['GUI:submodules','getzonefiles','getzonenames'],
+              );
 
 =head1 DESCRIPTION
 
@@ -214,11 +254,11 @@ provides access to B<QWizard> for DNSSEC-Tools software.
 In particular, the I<dnssec_tools_get_qwprimitives()> returns a set of
 primary screens for requesting a set of zone files followed by a set
 of domain names for those zone files.  These are then pushed into the
-@guiargs which you should treat as the final ARGV array to process.
+@guiargs which should be treated as the final ARGV array to process.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2010 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
