@@ -4,7 +4,7 @@
 # can do whatever you want with this stuff. If we meet some day, and you think
 # this stuff is worth it, you can buy me a beer in return.   Anton Berezin
 # ----------------------------------------------------------------------------
-# Copyright (c) 2005-2009 SPARTA, Inc.
+# Copyright (c) 2005-2010 SPARTA, Inc.
 # All rights reserved.
 #  
 # Redistribution and use in source and binary forms, with or without
@@ -33,7 +33,7 @@
 # OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 # ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
-# $Id: Fast.pm 4353 2009-02-06 20:57:28Z hardaker $
+# $Id: Fast.pm 4996 2010-01-24 18:14:06Z tewok $
 #
 package Net::DNS::ZoneFile::Fast;
 # documentation at the __END__ of the file
@@ -46,13 +46,13 @@ use Net::DNS;
 use Net::DNS::RR;
 use MIME::Base64;
 
-$VERSION = '1.1';
+$VERSION = '1.12';
 
 my $MAXIMUM_TTL = 0x7fffffff;
 
-my $pat_ttl = qr{[\dwdhms]+}i;
+my $pat_ttl = qr{\d+[\dwdhms]*}i;
 my $pat_skip = qr{\s*(?:;.*)?};
-my $pat_name = qr{[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*};
+my $pat_name = qr{[-\*\w\$\d\/*]+(?:\.[-\*\w\$\d\/]+)*};
 my $pat_maybefullname = qr{[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*\.?};
 my $pat_maybefullnameorroot = qr{(?:\.|[-\w\$\d\/*]+(?:\.[-\w\$\d\/]+)*\.?)};
 
@@ -80,7 +80,7 @@ my $includes_root;
 my $globalerror;
 my $nsec3capable;
 
-# boot strap optional DNSSEC module functcions
+# boot strap optional DNSSEC module functions
 # (not optional if trying to parse a signed zone, but we don't need
 # these modules unless we are.
 $nsec3capable = eval {
@@ -274,7 +274,7 @@ sub parse_line
 	  	error("\$GENERATE pattern without a wildcard") if $pat !~ /\$/;
 	  	while ($from <= $to) {
 	  		$_ = $pat;
-	  		s{\$ (?:\{ (\d+) (?:, (\d+) (?:, ([doxX]) )? )? \})?}
+	  		s{\$ (?:\{ ([\d+-]+) (?:, (\d+) (?:, ([doxX]) )? )? \})?}
 	  			{
 	  				my ($offset, $width, $base) = ($1, $2, $3);
 	  				$offset ||= 0;
@@ -573,12 +573,13 @@ sub parse_line
 		 };
 	  $parse->();
 	  return;
-      } elsif (/\G(txt)[ \t]+/igc) {
+      } elsif (/\G(txt|spf)[ \t]+/igc) {
+	  my $type = uc($1);
 	  if (/\G('[^']+')$pat_skip$/gc) {
 	      push @zone, {
 			   Line    => $ln,
 			   name    => $domain,
-			   type    => "TXT",
+			   type    => $type,
 			   ttl     => $ttl,
 			   class   => "IN",
 			   txtdata => $1,
@@ -587,7 +588,7 @@ sub parse_line
 	      push @zone, {
 			   Line    => $ln,
 			   name    => $domain,
-			   type    => "TXT",
+			   type    => $type,
 			   ttl     => $ttl,
 			   class   => "IN",
 			   txtdata => $1,
@@ -596,13 +597,13 @@ sub parse_line
 	      push @zone, {
 			   Line    => $ln,
 			   name    => $domain,
-			   type    => "TXT",
+			   type    => $type,
 			   ttl     => $ttl,
 			   class   => "IN",
 			   txtdata => $1,
 			  };
 	  } else {
-	      error("bad txtdata in TXT");
+	      error("bad txtdata in $type");
 	  }
       } elsif (/\G(sshfp)[ \t]+/igc) {
 	  if (/\G(\d+)\s+(\d+)\s+\(\s*$/gc) {
@@ -693,9 +694,8 @@ sub parse_line
 	      error("bad LOC data");
 	  }
       } elsif (/\G(hinfo)[ \t]+/igc) {
-	  # parsing stolen from Net::DNS::RR::HINFO
-	  if (/\G["'](.*?)["']\s+["'](.*?)["']$pat_skip$/gc) {
-	      push @zone, {
+	  if (/\G(["'].*?["']|\S+)\s+(["'].*?["']|\S+)$pat_skip$/gc) {
+	      my $result = {
 			   Line      => $ln,
 			   name      => $domain,
 			   type      => "HINFO",
@@ -704,6 +704,11 @@ sub parse_line
 			   cpu       => $1,
 			   os        => $2,
 			  };
+	      $result->{'cpu'} =~ s/^["']//;
+	      $result->{'cpu'} =~ s/["']$//;
+	      $result->{'os'} =~ s/^["']//;
+	      $result->{'os'} =~ s/["']$//;
+	      push @zone, $result;
 	  } else {
 	      error("bad HINFO data");
 	  }
@@ -745,7 +750,7 @@ sub parse_line
 	  if (/\G\(\s*$/gc) {
 	      # multi-line
 	      $parse = \&parse_rrsig;
-	  } elsif (/\G(\d+)\s+(\d+)\s+($pat_maybefullname)\s+([^=]+=)\s*/gc) {
+	  } elsif (/\G(\d+)\s+(\d+)\s+($pat_maybefullnameorroot)\s+([^=]+=)\s*/gc) {
 	      # single-line
 	      $rrsig->{'siginception'} = $1;
 	      $rrsig->{'keytag'} = $2;
@@ -813,7 +818,7 @@ sub parse_line
 	      error("bad DS data");
 	  }
       } elsif (/\G(nsec)[ \t]+/igc) {
-	  if (/\G\s*($pat_maybefullname)\s+(.*)$pat_skip$/gc) {
+	  if (/\G\s*($pat_maybefullnameorroot)\s+(.*)$pat_skip$/gc) {
 	      # XXX: set the typebm field ourselves?
 	      my ($nxtdname, $typelist) = ($1, $2);
 	      $typelist = join(" ",sort split(/\s+/,$typelist));
@@ -1027,7 +1032,7 @@ sub parse_rrsig
       # got more data
       if ($rrsig->{'first'}) {
 	  delete $rrsig->{'first'};
-	  if (/\G\s*(\d+)\s+(\d+)\s+($pat_maybefullname)/gc) {
+	  if (/\G\s*(\d+)\s+(\d+)\s+($pat_maybefullnameorroot)/gc) {
 	      $rrsig->{'siginception'} = $1;
 	      $rrsig->{'keytag'} = $2;
 	      $rrsig->{'signame'} = $3;
@@ -1175,8 +1180,8 @@ sub parse_soa_number
 	  }
 	  delete $soa->{nextkey};
 	  delete $soa->{breakable};
-	  chop $soa->{mname};
-	  chop $soa->{rname};
+	  $soa->{mname} =~ s/\.$//;
+	  $soa->{rname} =~ s/\.$//;
 	  $soa->{Lines} = $ln - $soa->{Line} + 1;
 	  push @zone, $soa;
 	  $soa = undef;
@@ -1365,7 +1370,7 @@ unless B<on_error> is also specified, in which case it is true.
 =item B<includes_root>
 
 An optional parameter.  By default, any $INCLUDE directives encountered
-will be tested for existance and readablility.  If the base path of the
+will be tested for existence and readability.  If the base path of the
 included filename is not your current working directory, this test will
 fail.  Set the B<includes_root> to the same as your named.conf file to
 avoid this failure.
@@ -1406,7 +1411,7 @@ either via hash lookup (C<$rr-E<gt>{Line}>), or via an accessor method
 
 =head1 BUGS
 
-The I<parse()> subroutine is not re-entrant.  Plobably will never be.
+The I<parse()> subroutine is not re-entrant, and it probably will never be.
 
 There is also no guarantee that I<parse()> will successfully parse every
 zone parsable by BIND, and no guarantee that BIND will parse every zone
@@ -1428,7 +1433,7 @@ Copyright 2003 by Anton Berezin and catpipe Systems ApS
 
   Anton Berezin
 
-Copyright (c) 2004-2008 SPARTA, Inc.
+Copyright (c) 2004-2010 SPARTA, Inc.
   All rights reserved.
    
   Redistribution and use in source and binary forms, with or without
@@ -1460,7 +1465,7 @@ Copyright (c) 2004-2008 SPARTA, Inc.
 =head1 CREDITS
 
 Anton Berezin created the versions up until 0.5.  Wes Hardaker at
-Sparta implemented the DNSSEC patches and took over maintaince of the
+Sparta implemented the DNSSEC patches and took over maintenance of the
 module from 0.6 onward.
 
 Anton's original CREDITS section:
