@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2009 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2010 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -32,14 +32,18 @@
 #
 #		set "signing-set-42"
 #			zonename	"example.com"
-#			keys		"Kexample.com.+005+87654 Kexample.com.+005+55555
+#			keys		"Kexample.com.+005+26000 Kexample.com.+005+55555
+#			keyrec_type	"set"
+#			set_type	"zskcur"
 #
 #		key "Kexample.com.+005+26000"
 #			zonename	"example.com"
-#			keyrec_type	"kskcur"
+#			keyrec_type	"zskcur"
 #			algorithm	"rsasha1"
-#			length		"1024"
+#			length		"2048"
 #			ksklife		"15768000"
+#                       revperiod       "3888000"
+#                       revtime         "1103277532"
 #			random		"-r /dev/urandom"
 #
 #	The current implementation assumes that only one keyrec file will be
@@ -54,12 +58,14 @@ require Exporter;
 use strict;
 use Net::DNS::SEC::Tools::conf;
 
-our $VERSION = "1.4";
+our $VERSION = "1.5";
 
 our @ISA = qw(Exporter);
 
-our @EXPORT = qw(keyrec_creat keyrec_open keyrec_read keyrec_fmtchk keyrec_names
-		 keyrec_fullrec keyrec_recval keyrec_setval keyrec_settime
+our @EXPORT = qw(keyrec_creat keyrec_open keyrec_read keyrec_fmtchk
+		 keyrec_names keyrec_fullrec
+		 keyrec_recval keyrec_setval keyrec_delval
+		 keyrec_settime keyrec_revoke_check
 		 keyrec_add keyrec_del keyrec_newkeyrec keyrec_exists
 		 keyrec_zonefields keyrec_setfields keyrec_keyfields
 		 keyrec_init keyrec_discard keyrec_close
@@ -74,7 +80,7 @@ our @EXPORT = qw(keyrec_creat keyrec_open keyrec_read keyrec_fmtchk keyrec_names
 #
 my @ZONEFIELDS = (
 			'zonefile',
-			'keyrec_type',		# Internal only.  Usually.
+			'keyrec_type',
 			'endtime',
 			'zskdirectory',
 			'kskdirectory',
@@ -83,6 +89,8 @@ my @ZONEFIELDS = (
 			'kskcount',
 			'kskcur',
 			'kskpub',
+			'kskrev',
+	                'rfc5011',
 			'signedzone',
 			'zskcount',
 			'zskcur',
@@ -100,7 +108,8 @@ my @ZONEFIELDS = (
 # Fields in a set keyrec.
 #
 my @SETFIELDS = (
-			'keyrec_type',		# Internal only.  Usually.
+			'keyrec_type',
+			'set_type',
 			'keys',
 		        'zonename',
 			'keyrec_setsecs',
@@ -117,6 +126,8 @@ my @KEYFIELDS = (
 			'keypath',		# Only set for obsolete ZSKs.
 			'ksklength',
 			'ksklife',
+			'revperiod',
+			'revtime',
 			'zsklength',
 			'zsklife',
 			'kgopts',
@@ -135,7 +146,6 @@ my $modified;				# File-modified flag.
 
 
 #--------------------------------------------------------------------------
-#
 # Routine:      keyrec_creat()
 #
 # Purpose:      Create a DNSSEC keyrec file, if it does not exist.  If
@@ -158,7 +168,6 @@ sub keyrec_creat
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:      keyrec_open()
 #
 # Purpose:      Opens an existing DNSSEC keyrec file.  If the file can't
@@ -186,7 +195,6 @@ sub keyrec_open
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_read()
 #
 # Purpose:	Read a DNSSEC keyrec file.  The contents are read into the
@@ -275,7 +283,7 @@ sub keyrec_read
 		# be enclosed in double quotes.
 		#
 #		$line =~ /^[ \t]*([a-zA-Z_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]+)"/;
-		$line =~ /^[ \t]*([a-zA-Z_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
+		$line =~ /^[ \t]*([a-zA-Z0-9_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
 		$keyword = $1;
 		$value = $2;
 #		print "keyrec_read:  keyword <$keyword>\t\t<$value>\n";
@@ -325,7 +333,6 @@ sub keyrec_read
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_fmtchk()
 #
 # Purpose:	Ensure the keyrec file is in the current format.
@@ -338,7 +345,7 @@ sub keyrec_read
 #					into a new kskcur entry.
 #				- keys:
 #					Ensure that each key in the zone is
-#					really pointing to a signing set.  if
+#					really pointing to a signing set.  If
 #					not, a new set is created and the key
 #					inserted in it.
 #
@@ -347,7 +354,8 @@ sub keyrec_read
 #
 #			key keyrecs:
 #				- ksk:
-#					Change to a kskcur, kskpub, or kskobs.
+#					Change to a kskcur, kskpub, kskrev,
+#					or kskobs.
 #
 sub keyrec_fmtchk()
 {
@@ -409,7 +417,7 @@ sub keyrec_fmtchk()
 		# pointing to a signing set.  If not, create a new
 		# set and move the key there.
 		#
-		foreach my $key (qw /kskcur kskpub zskcur zskpub zsknew/)
+		foreach my $key (qw /kskcur kskpub kskrev kskobs zskcur zskpub zsknew/)
 		{
 			my $keyname;		# Key's name.
 			my $set;		# Signing set name.
@@ -471,7 +479,7 @@ sub keyrec_fmtchk()
 
 	#
 	# Check key keyrecs for problems:
-	#	- Change ksk type keyrecs to kskcur, kskpub, or kskobs.
+	#	- Change ksk type keyrecs to kskcur, kskpub, kskrev, or kskobs.
 	#
 	foreach $krn (keyrec_names())
 	{
@@ -510,7 +518,7 @@ sub keyrec_fmtchk()
 			# any of the zone's key sets.
 			#
 			$zone = $krec{'zonename'};
-			foreach my $key (qw /kskcur kskpub/)
+			foreach my $key (qw /kskcur kskpub kskrev kskobs/)
 			{
 				$set = $keyrecs{$zone}{$key};
 				if(keyrec_signset_haskey($set,$krn))
@@ -527,11 +535,12 @@ sub keyrec_fmtchk()
 			#
 			if($found ne "")
 			{
-				keyrec_setval('key',$krn,'keyrec_type',$found);
+			  keyrec_setval('key',$krn,'keyrec_type',$found);
 			}
 			else
 			{
-				keyrec_setval('key',$krn,'keyrec_type','kskobs');
+			  keyrec_setval('key',$krn,'keyrec_type','kskobs');
+			  # XXX - shouldn't these be archived??
 			}
 			$changes++;
 		}
@@ -549,7 +558,86 @@ sub keyrec_fmtchk()
 }
 
 #--------------------------------------------------------------------------
+# Routine:	keyrec_revoke_check()
 #
+# Purpose:	Checks if the given KSK is within its revocation period or
+#		if it has fallen out of the revocation period.  The keyrec's
+#		revtime and revperiod are consulted.  If the revoked time
+#		subtracted from the current time exceeds the revocation, then
+#		the key should be moved from the revoked state to the obsolete
+#		state.
+#
+#		This routine performs the revocation check only!  It does not
+#		manipulate the key types itself; that must be done as needed
+#		by callers of this routine.
+#
+# Return Values:
+#		1  - returned if the key is outside the revocation period and
+#		     should be marked obsolete.
+#		0  - returned if the key is in the revocation period and
+#		     should be left revoked.
+#		-1 - returned on error
+#
+sub keyrec_revoke_check
+{
+	my $key = shift;			# Name of key to check.
+	my $keytype;				# Key's type.
+	my $revtime;				# Key's revoke time.
+	my $revperiod;				# Key's revoke period.
+	my $now = time();			# Current time.
+
+	#
+	# Get the key's type.
+	#
+	$keytype = keyrec_recval($key, 'keyrec_type');
+
+	#
+	# Ensure we were given a revoked KSK keyrec.
+	#
+	if($keytype ne 'kskrev')
+	{
+		err("keyrec_revoke_check:  $key: invalid type \"$keytype\"\n", -1);
+		return(-1);;
+	}
+
+	#
+	# Get the key's revocation information.
+	#
+	$revtime   = keyrec_recval($key, 'revtime');
+	$revperiod = keyrec_recval($key, 'revperiod');
+
+	#
+	# Ensure the key has the proper revocation data.
+	#
+	if(!defined($revtime))
+	{
+		err("keyrec_revoke_check:  $key: undefined revocation time\n", -1);
+		return(-1);
+	}
+
+	#
+	# Ensure the key has the proper revocation data.
+	#
+	if(!defined($revperiod))
+	{
+		err("keyrec_revoke_check:  $key: undefined revocation period\n", -1);
+		return(-1);
+	}
+
+	#
+	# If the key has exceeded its revocation period, we'll return a flag
+	# indicating that it is obsolete.  If it hasn't, we'll return a flag
+	# indicating it is still in revoked state.
+	#
+	if(($now - $revtime) > $revperiod)
+	{
+		return(1);
+	}
+
+	return(0);
+}
+
+#--------------------------------------------------------------------------
 # Routine:	keyrec_names()
 #
 # Purpose:	Smoosh the keyrec names into an array and return the array.
@@ -568,7 +656,6 @@ sub keyrec_names
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_keypaths()
 #
 # Purpose:	Return an array of the paths of a zone keyrec's keys.
@@ -591,8 +678,10 @@ sub keyrec_keypaths
 	#
 	# Ensure the keyrec type is valid.
 	#
-	if(($krt ne "kskcur") && ($krt ne "kskpub") && ($krt ne "ksknew") &&
-	   ($krt ne "zskcur") && ($krt ne "zskpub"))
+	if(($krt ne "kskcur") && ($krt ne "kskpub") &&
+	   ($krt ne "kskrev") && ($krt ne "kskobs") &&
+	   ($krt ne "zskcur") && ($krt ne "zskpub") &&
+	   ($krt ne "zsknew") && ($krt ne "zskobs"))
 	{
 		return(@paths);
 	}
@@ -633,7 +722,6 @@ sub keyrec_keypaths
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_exists()
 #
 # Purpose:	Smoosh the keyrec names into an array and return the array.
@@ -647,7 +735,6 @@ sub keyrec_exists
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_fullrec()
 #
 # Purpose:	Return all entries in a given keyrec.
@@ -661,7 +748,6 @@ sub keyrec_fullrec
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_recval()
 #
 # Purpose:	Return the value of a name/subfield pair.
@@ -676,7 +762,6 @@ sub keyrec_recval
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_setval()
 #
 # Purpose:	Set the value of a name/subfield pair.  The value is saved
@@ -695,6 +780,11 @@ sub keyrec_setval
 	my $krind;			# Loop index for finding keyrec.
 	my $lastfld = 0;		# Last found field in @keyreclines.
 
+	#
+	# Make sure we've got the correct count of keyrec lines.
+	#
+	$keyreclen = @keyreclines;
+
 	#
 	# If a keyrec of the specified name doesn't exist, we'll create a
 	# new one.  If the field is "keyrec_type", then we're creating a new
@@ -739,7 +829,7 @@ sub keyrec_setval
 		# Dig out the line's keyword and value.
 		#
 #		$line =~ /^[ \t]*([a-zA-Z_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]+)"/;
-		$line =~ /^[ \t]*([a-zA-Z_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
+		$line =~ /^[ \t]*([a-zA-Z0-9_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
 		$krtype = $1;
 		$krname = $2;
 
@@ -778,7 +868,7 @@ sub keyrec_setval
 		# Get the line's keyword and value.
 		#
 #		$line =~ /^[ \t]*([a-zA-Z_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]+)"/;
-		$line =~ /^[ \t]*([a-zA-Z_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
+		$line =~ /^[ \t]*([a-zA-Z0-9_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
 		$lkw = $1;
 		$lval = $2;
 
@@ -825,7 +915,6 @@ sub keyrec_setval
 	#
 	if($found)
 	{
-#		$keyreclines[$fldind] =~ s/"([a-zA-Z0-9\/\-+_.,: \t]+)"/"$val"/;
 		$keyreclines[$fldind] =~ s/"([a-zA-Z0-9\/\-+_.,: \t]*)"/"$val"/;
 	}
 	else
@@ -868,7 +957,6 @@ sub keyrec_setval
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_delval()
 #
 # Purpose:	Delete a name/subfield pair from a keyrec.  The value is
@@ -889,6 +977,11 @@ sub keyrec_delval
 	#
 	return(1) if(!exists($keyrecs{$name}));
 
+	#
+	# Make sure we've got the correct count of keyrec lines.
+	#
+	$keyreclen = @keyreclines;
+
 	#
 	# Find the appropriate entry to delete from @keyreclines.  If
 	# the given field isn't set in $name's keyrec, we'll return.
@@ -902,7 +995,7 @@ sub keyrec_delval
 		#
 		# Dig out the line's keyword and value.
 		#
-		$line =~ /^[ \t]*([a-zA-Z_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
+		$line =~ /^[ \t]*([a-zA-Z0-9_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
 		$krtype = $1;
 		$krname = $2;
 
@@ -940,7 +1033,7 @@ sub keyrec_delval
 		#
 		# Get the line's keyword and value.
 		#
-		$line =~ /^[ \t]*([a-zA-Z_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
+		$line =~ /^[ \t]*([a-zA-Z0-9_]+)[ \t]+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
 		$lkw = $1;
 		$lval = $2;
 
@@ -980,6 +1073,7 @@ sub keyrec_delval
 			splice @keyreclines, $fldind, 1;
 
 			$modified = 1;
+			$keyreclen--;
 			return(1);
 		}
 	}
@@ -991,7 +1085,6 @@ sub keyrec_delval
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_add()
 #
 # Purpose:	Adds a new keyrec and fields to %keyrecs and $keyreclines.
@@ -1049,6 +1142,11 @@ sub keyrec_add
 		$datestr = 'keyrec_gendate';
 	}
 
+	#
+	# Make sure we've got the correct count of keyrec lines.
+	#
+	$keyreclen = @keyreclines;
+
 	#
 	# Add the new keyrec's first line to the end of the keyrec table.
 	#
@@ -1085,8 +1183,8 @@ sub keyrec_add
 			#
 			if($fields{'keyrec_type'} !~ /^ksk/)
 			{
-				if(($fn eq 'ksklength')		||
-				   ($fn eq 'ksklife'))
+				if(($fn eq 'ksklength')	|| ($fn eq 'ksklife') ||
+				    ($fn eq 'revperiod') || ($fn eq 'revtime'))
 				{
 					next;
 				}
@@ -1144,7 +1242,6 @@ sub keyrec_add
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_del()
 #
 # Purpose:	Deletes a keyrec and fields from %keyrecs and $keyreclines.
@@ -1193,6 +1290,11 @@ sub keyrec_del
 		$krtype = "key";
 	}
 
+	#
+	# Make sure we've got the correct count of keyrec lines.
+	#
+	$keyreclen = @keyreclines;
+
 	#
 	# Find the index of the first line for this keyrec in the
 	# list of file lines.
@@ -1258,7 +1360,6 @@ sub keyrec_del
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_newkeyrec()
 #
 # Purpose:	Creates a keyrec in %keyrecs.  The name and type fields of
@@ -1284,7 +1385,6 @@ sub keyrec_newkeyrec
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_settime()
 #
 # Purpose:	Set the time value of a keyrec to the current time.  If
@@ -1327,7 +1427,6 @@ sub keyrec_settime
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_zonefields()
 #
 # Purpose:	Return the list of zone fields.
@@ -1338,7 +1437,6 @@ sub keyrec_zonefields
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_setfields()
 #
 # Purpose:	Return the list of set fields.
@@ -1349,7 +1447,6 @@ sub keyrec_setfields
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_keyfields()
 #
 # Purpose:	Return the list of key fields.
@@ -1380,7 +1477,7 @@ sub keyrec_signset_newname
 	# Get the zone's last signing set.
 	#
 	$setname = $keyrecs{$zone}{'lastset'};
-	$setname = "$zone-signset-0" if(!defined($keyrecs{$zone}{'lastset'}));
+	$setname = "$zone-signset-000" if(!defined($keyrecs{$zone}{'lastset'}));
 
 	#
 	# Get the first number in the set name.  If there isn't a number
@@ -1391,7 +1488,7 @@ sub keyrec_signset_newname
 	
 	if($oldind eq '')
 	{
-		$setname = $setname . "0";
+		$setname = $setname . "000";
 		$oldind = 0;
 	}
 
@@ -1400,6 +1497,11 @@ sub keyrec_signset_newname
 	# The set name in our pile o' options will be changed to the new name.
 	#
 	$newind = int($oldind) + 1;
+	while(length($newind) < 5)
+	{
+		$newind = "0$newind";
+	}
+
 	$setname =~ s/$setprefix$oldind/$setprefix$newind/;
 #	$keyrecs{$zone}{'lastset'} = $setname;
 	keyrec_setval('zone',$zone,'lastset',$setname);
@@ -1412,20 +1514,19 @@ sub keyrec_signset_newname
 
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_signset_new()
 #
 # Purpose:	Add a new signing set keyrec.   If the signing set keyrec
 #		hasn't yet been added with keyrec_add(), then we'll add it
-#		now.  The second through Nth arguments are concatenated
+#		now.  The third through Nth arguments are concatenated
 #		into a space-separated string and then that string is saved
-#		in both %keyrecs and in @keyreclines.  The $modified file-
-#		modified flag is updated, along with the length $keyreclen.
+#		in both %keyrecs and in @keyreclines.
 #
 sub keyrec_signset_new
 {
 	my $zone  = shift;		# Signing Set's zone.
 	my $name  = shift;		# Signing Set name we're creating.
+	my $type  = shift;		# Type of signing Set we're creating.
 
 	my $val;			# New value for the keyrec's subfield.
 	my $ret;			# Return code from keyrec_setval().
@@ -1438,6 +1539,17 @@ sub keyrec_signset_new
 		return(-1) if(keyrec_add('set',$name) < 0);
 	}
 
+	#
+	# Ensure the given set type is valid.
+	#
+	if(($type ne "kskcur") && ($type ne "kskpub") &&
+	   ($type ne "kskrev") && ($type ne "kskobs") &&
+	   ($type ne "zskcur") && ($type ne "zskpub") &&
+	   ($type ne "zsknew") && ($type ne "zskobs"))
+	{
+		return(-1);
+	}
+
 	#
 	# Bloodge together the remaining arguments into a single string.
 	# We'll use this for the signing set's set of keys.
@@ -1450,12 +1562,11 @@ sub keyrec_signset_new
 	#
 	$ret = keyrec_setval('set',$name,'zonename',$zone);
 	return($ret) if($ret != 0);
-	$ret = keyrec_setval('set',$name,'keys',$val);
+	$ret = keyrec_setval('set',$name,'set_type',$type);
 	return($ret);
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_signset_addkey()
 #
 # Purpose:	Add a key to a Signing Set for the specified keyrec.
@@ -1518,7 +1629,6 @@ sub keyrec_signset_addkey
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_signset_delkey()
 #
 # Purpose:	Delete an entry from a Signing Set for the specified keyrec.
@@ -1581,7 +1691,6 @@ sub keyrec_signset_delkey
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_signset_haskey()
 #
 # Purpose:	Check if a Signing Set contains the specified keyrec.
@@ -1629,7 +1738,6 @@ sub keyrec_signset_haskey
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_signset_clear()
 #
 # Purpose:	Delete all keys for the specified Signing Set.
@@ -1664,7 +1772,6 @@ sub keyrec_signset_clear
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_signsets()
 #
 # Purpose:	Return the names of the Signing Sets in the keyrec file.
@@ -1690,7 +1797,6 @@ sub keyrec_signsets
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_init()
 #
 # Purpose:	Initialize the internal data.
@@ -1704,7 +1810,6 @@ sub keyrec_init
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_defkrf()
 #
 # Purpose:	Get the default keyrec file defined in the DNSSEC-Tools
@@ -1729,7 +1834,6 @@ sub keyrec_defkrf
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_discard()
 #
 # Purpose:	Discard the current keyrec file -- don't save the contents,
@@ -1742,7 +1846,6 @@ sub keyrec_discard
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_close()
 #
 # Purpose:	Save the key record file and close the descriptor.
@@ -1754,7 +1857,6 @@ sub keyrec_close
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_write()
 #
 # Purpose:	Save the key record file and leave the file handle open.
@@ -1769,6 +1871,11 @@ sub keyrec_write
 	#
 	return if(!$modified);
 
+	#
+	# Make sure we've got the correct count of keyrec lines.
+	#
+	$keyreclen = @keyreclines;
+
 	#
 	# Loop through the array of keyrec lines and concatenate them all.
 	#
@@ -1793,7 +1900,6 @@ sub keyrec_write
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_saveas()
 #
 # Purpose:	Save the key record file into a user-specified file.  A new
@@ -1804,6 +1910,11 @@ sub keyrec_saveas
 	my $newname = shift;		# Name of new file.
 	my $krc = "";			# Concatenated keyrec file contents.
 
+	#
+	# Make sure we've got the correct count of keyrec lines.
+	#
+	$keyreclen = @keyreclines;
+
 	#
 	# Loop through the array of keyrec lines and concatenate them all.
 	#
@@ -1826,7 +1937,6 @@ sub keyrec_saveas
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_dump_hash()
 #
 # Purpose:	Dump the parsed keyrec entries.
@@ -1851,13 +1961,17 @@ sub keyrec_dump_hash
 }
 
 #--------------------------------------------------------------------------
-#
 # Routine:	keyrec_dump_array()
 #
 # Purpose:	Display the contents of @keyreclines.
 #
 sub keyrec_dump_array
 {
+	#
+	# Make sure we've got the correct count of keyrec lines.
+	#
+	$keyreclen = @keyreclines;
+
 	#
 	# Loop through the array of keyrec lines and print them all.
 	#
@@ -1905,9 +2019,11 @@ Net::DNS::SEC::Tools::keyrec - DNSSEC-Tools I<keyrec> file operations
 
   @kskpaths = keyrec_keypaths("example.com","kskcur");
 
+  $obsflag = keyrec_revoke_check("Kexample.com.+005+12345");
+
   $setname = keyrec_signset_newname("example.com");
 
-  keyrec_signset_new("zone","example-keys");
+  keyrec_signset_new($zone,"example-set-21","zskcur");
 
   keyrec_signset_addkey("example-keys","Kexample.com+005+12345",
  					 "Kexample.com+005+54321");
@@ -1954,8 +2070,10 @@ following is an example of a key I<keyrec>:
           keyrec_type     "kskcur"
           algorithm       "rsasha1"
           random          "/dev/urandom"
-          ksklength       "512"
+          ksklength       "2048"
 	  ksklife	  "15768000"
+	  revperiod	  "3888000"
+	  revtime	  "1103277532"
           keyrec_gensecs  "1101183727"
           keyrec_gendate  "Tue Nov 23 04:22:07 2004"
 
@@ -2079,6 +2197,18 @@ I<keyrec_name>.
 
 This routine returns a list of the recognized fields for a key I<keyrec>.
 
+=item I<keyrec_keypaths(zonename,keytype)>
+
+I<keyrec_keypaths()> returns a list of paths to a set of key files for a
+given zone.  The zone is specified in I<zonename> and the type of key is
+given in I<keytype>.
+
+I<keytype> must be one of the following:  "kskcur", "kskpub", "kskrev",
+"kskobs"", "zskcur", "zskpub", "zsknew", or "zskobs".
+
+If the given key type is not defined in the given zone's zone I<keyrec>, then
+a null set is returned.
+
 =item I<keyrec_names()>
 
 This routine returns a list of the I<keyrec> names from the file.
@@ -2126,6 +2256,25 @@ The call:
 
 will return the value "db.example.com".
 
+=item I<keyrec_revoke_check(key)>
+
+This interface checks a revoked KSK's I<keyrec> to determine if it is in or
+out of its revocation period.  The key must be a "kskrev" type key, and it
+must have "revtime" and "revperiod" fields defined in the I<keyrec>.
+
+The determination is made by subtracting the revoke time from the current
+time.  If this is greater than the revocation period, the the key has
+exceeded the time in which it must be revoked.  If not, then it must remain
+revoked.
+
+Return values:
+
+     1 specified key is outside the revocation period and should be
+       marked as obsolete
+     0 specified key is in the revocation period and should be left
+       revoked
+    -1 error (invalid key type, missing I<keyrec> data)
+
 =item I<keyrec_saveas(keyrec_file_copy)>
 
 This interface saves the internal version of the I<keyrec> file (opened with
@@ -2151,19 +2300,6 @@ Return values are:
     0 if the creation succeeded
     -1 invalid type was given
 
-
-=item I<keyrec_keypaths(zonename,keytype)>
-
-I<keyrec_keypaths()> returns a list of paths to a set of key files for a
-given zone.  The zone is specified in I<zonename> and the type of key is
-given in I<keytype>.
-
-I<keytype> must be one of the following:  "kskcur", "kskpub", "ksknew",
-"zskcur", or "zskpub".
-
-If the given key type is not defined in the given zone's zone I<keyrec>, then
-a null set is returned.
-
 =item I<keyrec_settime(keyrec_type,keyrec_name)>
 
 Set the timestamp of a specified I<keyrec>.  The file is B<not> written
@@ -2219,9 +2355,13 @@ determined:
     signing-88-set-1		  88
     signingset4321		4321
 
-=item I<keyrec_signset_new(signing_set_name)>
+=item I<keyrec_signset_new(zone,signing_set_name,set_type)>
+
+I<keyrec_signset_new()> creates the Signing Set named by I<signing_set_name>
+for the zone I<zone>.  It is given the type I<type>, which must be one of the
+following:  "kskcur", "kskpub", "kskrev", "kskobs", "zskcur", "zskpub",
+"zsknew", or "zskobs".
 
-I<keyrec_signset_new()> creates the Signing Set named by I<signing_set_name>.
 It returns 1 if the call is successful; 0 if it is not.
 
 =item I<keyrec_signset_addkey(signing_set_name,key_list)>
@@ -2317,7 +2457,7 @@ I<keyrec> file.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2010 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
