@@ -1,5 +1,5 @@
 #
-# Copyright 2005-2009 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2005-2010 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details
 #
 # DNSSEC Tools
@@ -28,6 +28,7 @@
 #			--------	---------
 #			FreeBSD		Unix
 #			Mac OSX		Unix
+#			Solaris		Solaris/Unix
 #
 #		When extending the interface or porting this module to another
 #		O/S, the following entities must be modified as described
@@ -95,7 +96,7 @@ use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::defaults;
 use Net::DNS::SEC::Tools::rolllog;
 
-our $VERSION = "0.91";
+our $VERSION = "1.5";
 
 our @ISA = qw(Exporter);
 
@@ -229,7 +230,7 @@ my $ROLLCMD_DSPUBALL	= "rollcmd_dspuball";
 my $ROLLCMD_GETSTATUS	= "rollcmd_getstatus";
 my $ROLLCMD_LOGFILE	= "rollcmd_logfile";
 my $ROLLCMD_LOGLEVEL	= "rollcmd_loglevel";
-my $ROLLCMD_LOGMSG	= "rollcmd_msg";
+my $ROLLCMD_LOGMSG	= "rollcmd_logmsg";
 my $ROLLCMD_ROLLALL	= "rollcmd_rollallzsks";
 my $ROLLCMD_ROLLKSK	= "rollcmd_rollksk";
 my $ROLLCMD_ROLLREC	= "rollcmd_rollrec";
@@ -356,6 +357,21 @@ my %switch_unix =
 	$SAVEID	   =>	\&unix_saveid,
 );
 
+my %switch_sysv =
+(
+	$CMDINT	   =>	\&unix_cmdint,
+	$DROPID	   =>	\&unix_psef_dropid,
+	$GETDIR	   =>	\&unix_dir,
+	$GETID	   =>	\&unix_getid,
+	$HALT	   =>	\&unix_halt,
+	$IDFILE	   =>	\&unix_idfile,
+	$SETIDFILE =>	\&unix_set_idfile,
+	$LOADZONE  =>	\&unix_loadzone,
+	$RMID	   =>	\&unix_rmid,
+	$RUNNING   =>	\&unix_running,
+	$SAVEID	   =>	\&unix_saveid,
+);
+
 
 ##############################################################################
 #
@@ -368,6 +384,7 @@ my %port_archs =
 	"uninitialized"	=>	\%switch_uninit,
 	"unknown"	=>	\%switch_unknown,
 	"unix"		=>	\%switch_unix,
+	"sysv"	 	=>	\%switch_sysv,
 );
 
 
@@ -376,7 +393,7 @@ my %port_archs =
 # Unix-related constants.
 # 
 
-my $UNIX_ROLLMGR_DIR	    = makelocalstatedir("run");
+my $UNIX_ROLLMGR_DIR	  = makelocalstatedir("run");
 our $UNIX_ROLLMGR_PIDFILE = ($UNIX_ROLLMGR_DIR . "/rollmgr.pid");
 
 my $PS = "/bin/ps";
@@ -391,30 +408,46 @@ my %switchtab = %{$port_archs{$osclass}};
 
 ##############################################################################
 #
-# These are textual descriptions of the rolling phases
+# These are textual descriptions of the rolling phases.
 # 
 my @zsk_roll_phases =
-  ('Not Rolling',
-   'Waiting for the old zone data to expire from caches',
-   'Signing the zone with the KSK and published ZSK',
-   'Waiting for the old zone data to expire from caches',
-   'Adjusting keys in the keyrec and signing the zone with new ZSK',
-  );
+(
+	'Not Rolling',
+	'Waiting for old zone data to expire from caches',
+	'Signing zone with KSK and Published ZSK',
+	'Waiting for old zone data to expire from caches',
+	'Adjusting keys in keyrec and signing zone with New ZSK',
+);
 
 my @ksk_roll_phases =
-  ('Not Rolling',
-   'Waiting for the old zone data to expire from caches',
-   'Generating a new published KSK',
-   'Waiting for cache or holddown timer expiration',
-   'Rolling the KSK(s)',
-   'Transfer the new KSK keyset to the parent',
-   'Waiting for the parent to publish the new DS record',
-   'Reloading the zone',
-  );
+(
+	'Not Rolling',
+	'Waiting for old zone data to expire from caches',
+	'Generating new Published KSK',
+	'Waiting for cache or holddown timer expiration',
+	'Rolling the KSK(s)',
+	'Transfer New KSK keyset to parent',
+	'Waiting for parent to publish new DS record',
+	'Reloading the zone',
+);
+
+my $MAXKSK = 7;					# Maximum KSK phase number.
+my $MAXZSK = 4;					# Maximum ZSK phase number.
 
 my %key_phases = 
-  ( 'KSK' => \@ksk_roll_phases,
-    'ZSK' => \@zsk_roll_phases );
+(
+	'KSK' => \@ksk_roll_phases,
+	'ZSK' => \@zsk_roll_phases
+);
+
+#
+# Description-length values for rollmgr_get_phase().
+#	NOTE:  These should be dealt with as configuration values.
+#	       However, logging will be changing RSN, and we'll wait a bit.
+#
+my $PHASELONG	= 1;				# Use long descriptions.
+my $PHASESHORT	= 0;				# Use short descriptions.
+my $longorshort = $PHASELONG;			# Description length to use.
 
 ##############################################################################
 ##############################################################################
@@ -457,6 +490,14 @@ sub rollmgr_prepdep
 		$osclass = "unix";
 	}
 
+	#
+	# Figure out which operating system class we're running on.
+	#
+	if(($osname eq "solaris"))
+	{
+		$osclass = "sysv";
+	}
+
 	#
 	# Get the appropriate switch table for this O/S class and save
 	# it for later reference.
@@ -1001,7 +1042,7 @@ sub unix_idfile
 #
 sub unix_set_idfile
 {
-    $UNIX_ROLLMGR_PIDFILE = $_[0];
+	$UNIX_ROLLMGR_PIDFILE = $_[0];
 }
 
 #--------------------------------------------------------------------------
@@ -1021,7 +1062,7 @@ sub unix_loadzone
 	#
 	# Get the path to the name server control program.
 	#
-	$rndc = dnssec_tools_default("bind_rndc") if($rndc eq "");
+	$rndc = dnssec_tools_default("rndc") if($rndc eq "");
 	return(0) if($rndc eq "");
 
 	#
@@ -1033,6 +1074,17 @@ sub unix_loadzone
 	return($ret);
 }
 
+#--------------------------------------------------------------------------
+#
+# Routine:	unix_psef_dropid()
+#
+# Purpose:      Replaces unix_dropid on ps -ef based systems
+#
+sub unix_psef_dropid
+{
+	return unix_dropid('-ps', '-ef', 1, @_);
+}
+
 #--------------------------------------------------------------------------
 #
 # Routine:	unix_dropid()
@@ -1040,16 +1092,28 @@ sub unix_loadzone
 # Purpose:	Ensures that another instance of rollerd is not running and
 #		then creates a pid file for future reference.
 #
+# Options:      [-ps FLAGS PIDPOSITION]
+#
 # Return Values:
 #		 1 - The pidfile was initialized for this process.
 #		 0 - Another process (not this one) is already acting as
 #		     rollerd.
+#		-1 - An error was encountered.
 #
 sub unix_dropid
 {
 	my $ego = $$;					# My identity.
 	my $pfpid;					# Pid from the pidfile.
 	my $pspid = -1;					# Pid from ps execution.
+	my $psflags = "wax";
+	my $pidposition = 0;
+
+	if ($_[0] eq '-ps')
+	{
+		shift @_;
+		$psflags = shift @_;
+		$pidposition = shift @_;
+	}
 
 # print "unix_dropid:  down in\n";
 
@@ -1061,13 +1125,19 @@ sub unix_dropid
 	#
 	# Create the file if it doesn't exist.
 	# If it does exist, we'll make sure the listed process isn't running.
+	# If we can't create it, we'll complain and return a failure code.
 	#
 	if($pfpid < 0)
 	{
 # print "unix_dropid:  opening $UNIX_ROLLMGR_PIDFILE\n";
 
 		unlink("$UNIX_ROLLMGR_PIDFILE");
-		open(PIDFILE,"> $UNIX_ROLLMGR_PIDFILE") || warn "DROPID UNABLE TO OPEN \"$UNIX_ROLLMGR_PIDFILE\"\n";
+		if(open(PIDFILE,"> $UNIX_ROLLMGR_PIDFILE") == 0)
+		{
+			warn "DROPID UNABLE TO OPEN \"$UNIX_ROLLMGR_PIDFILE\"\n";
+			return(-1);
+		}
+
 		flock(PIDFILE,LOCK_EX);
 	}
 	else
@@ -1090,7 +1160,7 @@ sub unix_dropid
 		#	However, the $pfpid seems to be dropped
 		#	when using that method.
 		#
-		$pscmd = "$PS wax";
+		$pscmd = "$PS $psflags";
 		$openrc = open(PSOUT,"$pscmd |");
 		$psline = <PSOUT>;
 		while(<PSOUT>)
@@ -1106,8 +1176,8 @@ sub unix_dropid
 			# Get the pid from the line and drop out.
 			#
 			s/^[ ]*//;
-			@psout = split / /;
-			$pspid = $psout[0];
+			@psout = split / +/;
+			$pspid = $psout[$pidposition];
 			last;
 		}
 		close(PSOUT);
@@ -1147,6 +1217,7 @@ sub unix_dropid
 	return(1);
 }
 
+
 #--------------------------------------------------------------------------
 #
 # Routine:	unix_rmid()
@@ -1449,6 +1520,7 @@ sub rollmgr_channel
 		#
 		$unixsock = makelocalstatedir("/dnssec-tools") . $UNIXSOCK;
 # print STDERR "rollmgr_channel:  unixsock - <$unixsock>\n";
+
 		$sockdata = sockaddr_un($unixsock);
 
 		#
@@ -1474,6 +1546,7 @@ sub rollmgr_channel
 			#
 			socket(CLNTSOCK,PF_UNIX,SOCK_STREAM,0)	|| return(-1);
 			connect(CLNTSOCK,$sockdata)		|| return(0);
+
 		}
 	}
 
@@ -1489,7 +1562,6 @@ sub rollmgr_channel
 #
 sub rollmgr_closechan
 {
-
 # print "rollmgr_closechan:  down in\n";
 	close(CLNTSOCK);
 }
@@ -1498,35 +1570,37 @@ sub rollmgr_closechan
 #
 # Routine:	rollmgr_queuecmd()
 #
-# Purpose:      This routine can be called internally to queue a command
-#          	for later processing via calls to rollmgr_getcmd().
-#          	It is useful when doing initial startup before full
-#          	processing is to commence.  Commands queued by this
-#          	process take precidence over commands received via the
-#          	command intervace (ie, via rollmgr_sendcmd()).
+# Purpose:	This routine can be called internally to queue a command
+#		for later processing via calls to rollmgr_getcmd().
+#		It is useful when doing initial startup before full
+#		processing is to commence.  Commands queued by this
+#		process take precedence over commands received via the
+#		command intervace (ie, via rollmgr_sendcmd()).
 #
 sub rollmgr_queuecmd
 {
-    my ($cmd, $value) = @_;
-    return 0 if (rollmgr_verifycmd($cmd) == 0);
-    push @queuedcmds, [$cmd, $value];
+	my ($cmd, $value) = @_;
+
+	return(0) if (rollmgr_verifycmd($cmd) == 0);
+	push @queuedcmds, [$cmd, $value];
 }
 
 #-----------------------------------------------------------------------------
 #
 # Routine:	rollmgr_getqueueitem()
 #
-# Purpose:      This routine can be called pull a command from the queue
-#               This is intended to process the the item, so it is removed
-#               from the queue.
+# Purpose:	This routine can be called pull a command from the queue
+#		This is intended to process the item, so it is removed
+#		from the queue.
 #
 sub rollmgr_getqueueitem
 {
-    if ($#queuedcmds > -1) {
-	my $cmd = shift @queuedcmds;
-	return $cmd;
-    }
-    return;
+	if($#queuedcmds > -1)
+	{
+		my $cmd = shift @queuedcmds;
+		return($cmd);
+	}
+	return;
 }
 
 
@@ -1534,16 +1608,17 @@ sub rollmgr_getqueueitem
 #
 # Routine:	rollmgr_getallqueuedcmds()
 #
-# Purpose:      This routine returns all the queued commands in the stack
-#               The items are left in place unless an truthful argument
-#               (e.g. "1") is passed in.
+# Purpose:	This routine returns all the queued commands in the stack
+#		The items are left in place unless an truthful argument
+#		(e.g. "1") is passed in.
 #
 sub rollmgr_getallqueuedcmds
 {
-    my $removefromqueue = shift;
-    my @results = @queuedcmds;
-    @queuedcmds = () if ($removefromqueue);
-    return @queuedcmds;
+	my $removefromqueue = shift;
+	my @results = @queuedcmds;
+
+	@queuedcmds = () if ($removefromqueue);
+	return(@queuedcmds);
 }
 
 
@@ -1573,11 +1648,11 @@ sub rollmgr_getcmd
 	# if we have anything queued up, process those first.
 	#
 	my $cmdandvalue = rollmgr_getqueueitem();
-	return (@$cmdandvalue)
-	  if (defined($cmdandvalue) && ref($cmdandvalue) eq 'ARRAY');
+	return(@$cmdandvalue)
+	    if(defined($cmdandvalue) && (ref($cmdandvalue) eq 'ARRAY'));
 
 	#
-	# set a time limit on how long we'll wait for the connection.
+	# Set a time limit on how long we'll wait for the connection.
 	# Our alarm handler is a dummy, only intended to keep us from
 	# waiting forever.
 	#
@@ -1798,14 +1873,41 @@ sub rollmgr_verifycmd
 # Routine:	rollmgr_get_phase()
 #
 # Purpose:	This routine translates the numerical phases of rolling
-#               keys into textual strings that are better descriptions
-#               for human consumption.
+#		keys into textual strings that are better descriptions
+#		for human consumption.
 #
 sub rollmgr_get_phase
 {
-    my ($keytype, $keyphase) = @_;
-    return '' if ($keyphase !~ /^\d$/); # must be a single digit number
-    return $key_phases{uc($keytype)}[$keyphase];
+	my ($keytype, $keyphase) = @_;
+	my $maxphase = -1;				# Maximum phase value.
+
+	#
+	# Don't return anything if the installer doesn't want long descritions.
+	#	NOTE:  This really should be a config value.
+	#
+	if($longorshort != $PHASELONG)
+	{
+		return('');
+	}
+
+	#
+	# Figure out the maximum phase for this key type.
+	#
+	$maxphase = $MAXKSK if($keytype eq 'KSK');
+	$maxphase = $MAXZSK if($keytype eq 'ZSK');
+
+	#
+	# Ensure the phase is within range and is a single digit.
+	#
+	if(($keyphase < 0) || ($keyphase > $maxphase) || ($keyphase !~ /^\d$/))
+	{
+		return('');
+	}
+
+	#
+	# Return the phase description.
+	#
+	return($key_phases{uc($keytype)}[$keyphase]);
 }
 
 1;
@@ -1848,6 +1950,8 @@ manager.
   rollmgr_sendcmd(CHANNEL_WAIT,ROLLCMD_ROLLZSK,"example.com");
   ($retcode, $respmsg) = rollmgr_getresp();
 
+  $descr = rollmgr_get_phase('KSK', $phasecnt);
+
 =head1 DESCRIPTION
 
 The B<Net::DNS::SEC::Tools::rollmgr> module provides standard,
@@ -1890,6 +1994,7 @@ Return Values:
 
     1 - the id file was successfully created for this process
     0 - another process is already acting as rollerd
+   -1 - unable to create the id file
 
 =item I<rollmgr_rmid()>
 
@@ -2036,6 +2141,17 @@ specific to the command sent.
 I<rollmgr_verifycmd()> verifies that I<cmd> is a valid command for B<rollerd>.
 1 is returned for a valid command; 0 is returned for an invalid command.
 
+1 is returned for a valid command; 0 is returned for an invalid command.
+
+=item I<rollmgr_get_phase(phasetype, phasenum)>
+
+I<rollmgr_get_phase()> returns a description of a particular phase for a
+particular type of rollover.  I<phasetype> specifies the type of rollover,
+and may be "KSK" or "ZSK".  I<phasenum> specifies the phase number whose
+description is desired.  This must be an integer between 0 and 7 (KSK) or 0
+and 4 (ZSK).  If an invalid phase type or phase number is specified, an empty
+string is returned. 
+
 =back
 
 =head1 WARNINGS
@@ -2048,7 +2164,7 @@ the file prior to opening it, but we can't do so without it being open.
 
 =head1 COPYRIGHT
 
-Copyright 2005-2009 SPARTA, Inc.  All rights reserved.
+Copyright 2005-2010 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
