@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# Copyright 2006-2009 SPARTA, Inc.  All rights reserved.  See the COPYING
+# Copyright 2006-2010 SPARTA, Inc.  All rights reserved.  See the COPYING
 # file distributed with this software for details.
 #
 # DNSSEC Tools
@@ -15,13 +15,13 @@
 #	the value enclosed in quotes.  Comments may be included by prefacing
 #	them with the '#' or ';' comment characters.
 #
-#	The format and contents of a rollrec file are *very* preliminary.
 #	These entries are grouped by the zone whose key(s) are being rolled
 #	over.
 #
 #	An example rollrec file follows:
 #
 #	    roll "example.com"
+#		zonename	"example.com"
 #		zonefile	"/usr/etc/dnssec-tools/zones/db.example.com"
 #		keyrec		"/usr/etc/dnssec-tools/keyrec/example.keyrec"
 #		zskphase	"2"
@@ -31,6 +31,7 @@
 #		phasestart	"Wed Mar 09 21:49:22 2005"
 #
 #	    roll "example2.com"
+#		zonename	"example2.com"
 #		zonefile	"/usr/etc/dnssec-tools/zone/db.example2.com"
 #		keyrec		"/usr/etc/dnssec-tools/keyrec/example2.keyrec"
 #		kskphase	"1"
@@ -61,7 +62,7 @@ use Fcntl qw(:DEFAULT :flock);
 use Net::DNS::SEC::Tools::conf;
 use Net::DNS::SEC::Tools::rollmgr;
 
-our $VERSION = "0.9";
+our $VERSION = "1.5";
 
 our @ISA = qw(Exporter);
 
@@ -103,6 +104,7 @@ my $LOCKNAME = "rollrec.lock";
 # Valid fields in a rollrec.
 #
 my @ROLLFIELDS = (
+			'zonename',
 			'zonefile',
 			'keyrec',
 			'maxttl',
@@ -151,8 +153,7 @@ sub rollrec_lock
 	#
 	# Get the DNSSEC-Tools config directory.
 	#
-	$lockdir = makelocalstatedir("/dnssec-tools")
-	  || $DEFAULT_DNSSECTOOLS_DIR;
+	$lockdir = makelocalstatedir("/dnssec-tools") || $DEFAULT_DNSSECTOOLS_DIR;
 
 	#
 	# Build our lock file.
@@ -287,7 +288,7 @@ sub rollrec_read
 		# and a number of punctuation characters.  The value *must*
 		# be enclosed in double quotes.
 		#
-		$line =~ /^\s*([a-zA-Z_]+)\s+"([a-zA-Z0-9\/\-+_.,: \t]*)"/;
+		$line =~ /^\s*([a-zA-Z_]+)\s+"([a-zA-Z0-9\/\-+_.,: \@\t]*)"/;
 		$keyword = $1;
 		$value = $2;
 #		print "rollrec_read:  keyword <$keyword>\t\t<$value>\n";
@@ -1285,10 +1286,11 @@ A I<rollrec> file is organized in sets of I<rollrec> records.  I<rollrec>s
 describe the state of a rollover operation.  A I<rollrec> consists of a set
 of keyword/value entries.  The following is an example of a I<rollrec>:
 
-    roll "example.com"
-	zonefile		"/usr/etc/dnssec-tools/zones/db.example.com"
-	keyrec			"/usr/etc/dnssec-tools/keyrec/example.keyrec"
-	directory		"/usr/etc/dnssec-tools/dir-example.com"
+    roll "example"
+	zonename		"example.com"
+	zonefile		"/etc/dnssec-tools/zones/db.example.com"
+	keyrec			"/etc/dnssec-tools/keyrec/example.keyrec"
+	directory		"/etc/dnssec-tools/dir-example.com"
 	kskphase		"0"
 	zskphase		"2"
 	maxttl			"86400"
@@ -1302,13 +1304,14 @@ of keyword/value entries.  The following is an example of a I<rollrec>:
         istrustanchor           "yes"
         holddowntime            "8W"
 
-Additionally cmds to be acted upon at start up can be defined using
-the "cmd" token as shown in the following example.  Use this feature
-with caution and only if you understand the internals of the
-application.
+Additionally, commands to be acted upon at start-up can be defined using the
+"cmd" token as shown in the following example.
 
     cmd "rollzsk example.com"
 
+Use this feature with caution and only if you understand the internals of
+B<rollerd> and I<exactly> what will be done by the specified command.
+
 The first step in using this module must be to read the I<rollrec> file.  The
 I<rollrec_read()> interface reads the file and parses it into an internal
 format.  The file's records are copied into a hash table (for easy reference
@@ -1589,7 +1592,7 @@ preserved as given in the I<rollrec> file.
 
 =head1 COPYRIGHT
 
-Copyright 2004-2009 SPARTA, Inc.  All rights reserved.
+Copyright 2006-2010 SPARTA, Inc.  All rights reserved.
 See the COPYING file included with the DNSSEC-Tools package for details.
 
 =head1 AUTHOR
