@@ -6,8 +6,8 @@
 #define VALIDATOR_DRIVER_H
 
 void print_val_response(struct val_response *resp);
-int sendquery(val_context_t * context, const char *desc, u_char * name_n,
-              const u_int16_t class, const u_int16_t type, u_int32_t flags,
+int sendquery(val_context_t * context, const char *desc, char * name,
+              int class, int type, u_int32_t flags,
               const int *result_ar, int trusted_only,
               struct val_response *resp);
 
