@@ -27,6 +27,10 @@
 #include <sys/types.h>
 #include <sys/time.h>
 
+#ifdef __cplusplus
+extern          "C" {
+#endif
+
 #ifdef MEMORY_DEBUGGING
 #define MALLOC(s) my_malloc(s, __FILE__, __LINE__)
 #define FREE(p) my_free(p,__FILE__,__LINE__)
@@ -38,7 +42,12 @@
 #endif
 
 
-#define RES_RETRY 0
+#define RES_RETRY 1 /* number of times to retry */
+#define LIBSRES_NS_STAGGER 5 /* how far apart should we stagger queries to
+                                different authoritative name servers */
+#ifndef RES_TIMEOUT
+#define RES_TIMEOUT 5 /* min seconds between retries */
+#endif
 
 #ifndef RES_USE_DNSSEC
 #define RES_USE_DNSSEC  0x00200000
@@ -107,6 +116,25 @@
 #define ns_t_ds       43
 #endif
 
+/* Keep the following consistent with validator.h */
+/* 
+ *  we are duplicating the following since there's no reason
+ *  for files in libsres that need these definitions to include
+ *  the complete validator.h file
+ */
+#ifndef SHA_DIGEST_LENGTH
+#define SHA_DIGEST_LENGTH   20
+#endif
+#ifndef SHA256_DIGEST_LENGTH
+#define SHA256_DIGEST_LENGTH 32
+#endif
+#ifndef ALG_DS_HASH_SHA1
+#define ALG_DS_HASH_SHA1 1
+#endif
+#ifndef ALG_DS_HASH_SHA256
+#define ALG_DS_HASH_SHA256 2
+#endif
+
 /* query types for which edns0 is required */
 #ifdef LIBVAL_DLV
 #define DNSSEC_METADATA_QTYPE(type) \
@@ -144,7 +172,7 @@
 #define SR_REFUSED                14    /*RCODE set to REFUSED */
 
 struct name_server {
-    u_int8_t        ns_name_n[NS_MAXCDNAME];
+    u_char          ns_name_n[NS_MAXCDNAME];
     void           *ns_tsig;
     u_int32_t       ns_security_options;
     u_int32_t       ns_status;
@@ -178,7 +206,10 @@ int             response_recv(int *trans_id,
                               fd_set *pending_desc,
                               struct timeval *closest_event,
                               struct name_server **respondent,
-                              u_int8_t ** answer, u_int * answer_length);
+                              u_char ** answer, size_t * answer_length);
+void            res_cancel(int *transaction_id);
+int             res_skipns(int transaction_id, struct timeval *closest_event);
+
 void            wait_for_res_data(fd_set * pending_desc, 
                                   struct timeval *closest_event);
 int             get(const char *name_n,
@@ -187,8 +218,8 @@ int             get(const char *name_n,
                     struct name_server *nslist,
                     int edns0_size,
                     struct name_server **server,
-                    u_int8_t ** response, u_int * response_length);
-void            print_response(u_int8_t * ans, int resplen);
+                    u_char ** response, size_t * response_length);
+void            print_response(u_char * ans, size_t resplen);
 int             clone_ns(struct name_server **cloned_ns,
                          struct name_server *ns);
 int             clone_ns_list(struct name_server **ns_list,
@@ -222,10 +253,22 @@ u_int16_t       res_nametotype(const char *buf, int *successp);
 const char     *p_section(int section, int opcode);
 #endif
 
-const char     *p_class(int class);
+const char     *p_class(int pclass);
 
 #undef p_type
 #define p_type(type) p_sres_type(type)
 const char     *p_sres_type(int type);
 
+/*
+ * at one open ns_msg_getflag was a macro on Linux, but now it is a
+ * function in libresolv. redifine to use our internal version.
+ */
+#ifndef ns_msg_getflag
+#define ns_msg_getflag libsres_msg_getflag
+#endif
+
+
+#ifdef __cplusplus
+}                               /* extern C */
+#endif
 #endif                          /* RESOLVER_H */
