@@ -14,9 +14,9 @@ extern          "C" {
 #endif
 
     struct query_list {
-        u_int8_t        ql_name_n[NS_MAXCDNAME];
-        u_int8_t        ql_zone_n[NS_MAXCDNAME];
-        u_int16_t       ql_type_h;
+        u_char        ql_name_n[NS_MAXCDNAME];
+        u_char        ql_zone_n[NS_MAXCDNAME];
+        u_int16_t     ql_type_h;
         struct query_list *ql_next;
     };
 
@@ -24,8 +24,10 @@ extern          "C" {
         struct query_list *queries;
         struct qname_chain *qnames;
         struct rrset_rec *answers;
+        struct name_server *cur_pending_glue_ns;
         struct name_server *pending_glue_ns;
         struct name_server *merged_glue_ns;
+        u_char             *saved_zonecut_n;
         struct rrset_rec *learned_zones;
     };
 
@@ -47,7 +49,7 @@ extern          "C" {
         u_int32_t       qc_flags;
         u_int32_t       qc_ttl_x;    /* ttl expiry time */
         int             qc_bad; /* contains "bad" data */
-        u_int8_t       *qc_zonecut_n;
+        u_char         *qc_zonecut_n;
 
         struct delegation_info *qc_referral;
         struct name_server *qc_ns_list;
@@ -103,8 +105,7 @@ extern          "C" {
         global_opt_t *g_opt;
         val_log_t *val_log_targets;
         
-        /* Query and authentication chain caches */
-        struct val_digested_auth_chain *a_list;
+        /* Query cache */
         struct val_query_chain *q_list;
     }; 
 
@@ -114,15 +115,38 @@ extern          "C" {
         struct val_digested_auth_chain *val_ac_next;
     };
 
-    struct val_digested_auth_chain {
-        val_astatus_t   val_ac_status;
-        union {
-            struct val_rrset_rec *val_ac_rrset;
-            struct val_rrset_digested _as;
-        };
-        struct val_query_chain *val_ac_query;
+    struct rrset_rec {
+        int       rrs_rcode;
+        u_char   *rrs_name_n;       /* Owner */
+        u_int16_t rrs_class_h;      /* ns_c_... */
+        u_int16_t rrs_type_h;       /* ns_t_... */
+        u_int32_t rrs_ttl_h;        /* Received ttl */
+        u_int32_t rrs_ttl_x;        /* ttl expire time */
+        u_int8_t  rrs_section;      /* VAL_FROM_... */
+        struct sockaddr *rrs_server;      /* respondent server */
+        struct val_rr_rec  *rrs_data; /* All data RR's */
+        struct val_rr_rec  *rrs_sig;  /* All signatures */
+        u_char *rrs_zonecut_n;
+        u_int8_t rrs_cred;       /* SR_CRED_... */
+        u_int8_t rrs_ans_kind;   /* SR_ANS_... */
+        struct rrset_rec *rrs_next;
     };
 
+    struct domain_info {
+        char           *di_requested_name_h;
+        u_int16_t       di_requested_type_h;
+        u_int16_t       di_requested_class_h;
+        struct rrset_rec *di_answers;
+        struct rrset_rec *di_proofs;
+        struct qname_chain *di_qnames;
+        int             di_res_error;
+    };
+
+    struct val_digested_auth_chain {
+        val_astatus_t   val_ac_status;
+        struct val_rrset_digested val_ac_rrset;
+        struct val_query_chain *val_ac_query;
+    };
 
     struct queries_for_query {
         u_int32_t qfq_flags;
