@@ -69,15 +69,13 @@ void            my_free(void *p, char *filename, int lineno);
 void           *my_malloc(size_t t, char *filename, int lineno);
 char           *my_strdup(const char *str, char *filename, int lineno);
 
-void            print_response(u_int8_t * ans, int resplen);
-void            print_hex_field(u_int8_t field[], int length, int width,
+void            print_response(u_char * ans, size_t resplen);
+void            print_hex_field(u_char field[], size_t length, size_t width,
                                 char *pref);
-void            print_hex(u_int8_t field[], int length);
-int             complete_read(int sock, void *field, int length);
-void            dump_response(const u_int8_t * ans, int resplen);
-int             wire_to_ascii_name(char *name, u_int8_t * wire,
-                                   int name_length);
-u_int16_t       retrieve_type(const u_int8_t * rr);
+void            print_hex(u_char field[], size_t length);
+int             complete_read(int sock, void *field, size_t length);
+void            dump_response(const u_char * ans, size_t resplen);
+u_int16_t       retrieve_type(const u_char * rr);
 u_int16_t       libsres_random(void);
 int             libsres_msg_getflag(ns_msg han, int flag);
 
