@@ -34,6 +34,23 @@
 
 #endif /*VAL_NO_THREADS*/
 
+
+struct nsecprooflist {
+    struct rrset_rec *the_set;
+    struct val_internal_result *res;
+    struct nsecprooflist *next;
+};
+#ifdef LIBVAL_NSEC3
+struct nsec3prooflist {
+    val_nsec3_rdata_t nd;
+    size_t nsec3_hashlen;
+    u_char *nsec3_hash;
+    struct val_internal_result *res;
+    struct rrset_rec *the_set;
+    struct nsec3prooflist *next;
+};
+#endif
+
 int             add_to_qfq_chain(val_context_t *context,
                                  struct queries_for_query **queries,
                                  u_char * name_n, const u_int16_t type_h,
@@ -43,23 +60,18 @@ int             add_to_qfq_chain(val_context_t *context,
 void            free_authentication_chain(struct val_digested_auth_chain
                                           *assertions);
 void            free_query_chain(struct val_query_chain *queries);
+void            free_query_chain_structure(struct val_query_chain *queries);
 void            requery_with_edns0(val_context_t *context,
                                     struct val_query_chain *matched_q);
-void            zap_query(val_context_t *context, struct val_query_chain *added_q);
-int             val_istrusted(val_status_t val_status);
-int             val_isvalidated(val_status_t val_status);
-int             val_does_not_exist(val_status_t status); 
-int             get_zse(val_context_t * ctx, u_int8_t * name_n, 
-                        u_int32_t flags, u_int16_t *status, u_int8_t ** match_ptr, u_int32_t *ttl_x);
-int             find_trust_point(val_context_t * ctx, u_int8_t * zone_n, 
-                                 u_int8_t ** matched_zone, u_int32_t *ttl_x);
-void            val_free_result_chain(struct val_result_chain *results);
+int             get_zse(val_context_t * ctx, u_char * name_n, 
+                        u_int32_t flags, u_int16_t *status, u_char ** match_ptr, u_int32_t *ttl_x);
+int             find_trust_point(val_context_t * ctx, u_char * zone_n, 
+                                 u_char ** matched_zone, u_int32_t *ttl_x);
 #ifdef LIBVAL_DLV
 int             check_anc_proof(val_context_t *context,
                                 struct val_query_chain *q, 
                                 u_int32_t flags,
-                                u_int8_t *name_n, 
-                                int check_wildcard,
+                                u_char *name_n, 
                                 int *matches);
 #endif
 int             try_chase_query(val_context_t * context,
@@ -70,11 +82,5 @@ int             try_chase_query(val_context_t * context,
                                 struct queries_for_query **queries,
                                 struct val_result_chain **results,
                                 int *done);
-int             val_resolve_and_check(val_context_t * context,
-                                      u_char * domain_name,
-                                      const u_int16_t class,
-                                      const u_int16_t type,
-                                      const u_int32_t flags,
-                                      struct val_result_chain **results);
 
 #endif
