@@ -33,6 +33,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 
+#include <validator/resolver.h>
 #include <validator/validator.h>
 #include <validator/validator-internal.h>
 #include "val_crypto.h"
@@ -43,28 +44,36 @@
  * Returns VAL_NO_ERROR on success, other values on failure 
  */
 static int
-dsasha1_parse_public_key(const unsigned char *buf, int buflen, DSA * dsa)
+dsasha1_parse_public_key(const u_char *buf, size_t buflen, DSA * dsa)
 {
     u_int8_t        T;
     int             index = 0;
     BIGNUM         *bn_p, *bn_q, *bn_g, *bn_y;
 
-    if (!dsa) {
+    if (!dsa || buflen == 0) {
         return VAL_BAD_ARGUMENT;
     }
 
     T = (u_int8_t) (buf[index]);
     index++;
-
+    
+    if (index+20 > buflen)
+        return VAL_BAD_ARGUMENT;
     bn_q = BN_bin2bn(buf + index, 20, NULL);
     index += 20;
 
+    if (index+64 + (T * 8) > buflen)
+        return VAL_BAD_ARGUMENT;
     bn_p = BN_bin2bn(buf + index, 64 + (T * 8), NULL);
     index += (64 + (T * 8));
 
+    if (index+64 + (T * 8) > buflen)
+        return VAL_BAD_ARGUMENT;
     bn_g = BN_bin2bn(buf + index, 64 + (T * 8), NULL);
     index += (64 + (T * 8));
 
+    if (index+64 + (T * 8) > buflen)
+        return VAL_BAD_ARGUMENT;
     bn_y = BN_bin2bn(buf + index, 64 + (T * 8), NULL);
     index += (64 + (T * 8));
 
@@ -78,16 +87,16 @@ dsasha1_parse_public_key(const unsigned char *buf, int buflen, DSA * dsa)
 
 void
 dsasha1_sigverify(val_context_t * ctx,
-                  const unsigned char *data,
-                  int data_len,
+                  const u_char *data,
+                  size_t data_len,
                   const val_dnskey_rdata_t * dnskey,
                   const val_rrsig_rdata_t * rrsig,
                   val_astatus_t * key_status, val_astatus_t * sig_status)
 {
     char            buf[1028];
-    int             buflen = 1024;
+    size_t          buflen = 1024;
     DSA            *dsa = NULL;
-    unsigned char   sha1_hash[SHA_DIGEST_LENGTH];
+    u_char   sha1_hash[SHA_DIGEST_LENGTH];
 
     val_log(ctx, LOG_DEBUG,
             "dsasha1_sigverify(): parsing the public key...");
@@ -109,7 +118,7 @@ dsasha1_sigverify(val_context_t * ctx,
     }
 
     bzero(sha1_hash, SHA_DIGEST_LENGTH);
-    SHA1(data, data_len, (unsigned char *) sha1_hash);
+    SHA1(data, data_len, sha1_hash);
     val_log(ctx, LOG_DEBUG, "dsasha1_sigverify(): SHA-1 hash = %s",
             get_hex_string(sha1_hash, SHA_DIGEST_LENGTH, buf, buflen));
 
@@ -117,8 +126,8 @@ dsasha1_sigverify(val_context_t * ctx,
             "dsasha1_sigverify(): verifying DSA signature...");
 
     if (DSA_verify
-        (NID_sha1, (unsigned char *) sha1_hash, SHA_DIGEST_LENGTH,
-         rrsig->signature, rrsig->signature_len, dsa)) {
+        (NID_sha1, (u_char *) sha1_hash, SHA_DIGEST_LENGTH,
+         rrsig->signature, rrsig->signature_len, dsa)  == 1) {
         val_log(ctx, LOG_INFO, "dsasha1_sigverify(): returned SUCCESS");
         DSA_free(dsa);
         *sig_status = VAL_AC_RRSIG_VERIFIED;
@@ -134,7 +143,7 @@ dsasha1_sigverify(val_context_t * ctx,
  * Returns VAL_NO_ERROR on success, other values on failure 
  */
 static int
-rsamd5_parse_public_key(const unsigned char *buf, int buflen, RSA * rsa)
+rsamd5_parse_public_key(const u_char *buf, size_t buflen, RSA * rsa)
 {
     int             index = 0;
     const u_char   *cp;
@@ -142,24 +151,24 @@ rsamd5_parse_public_key(const unsigned char *buf, int buflen, RSA * rsa)
     BIGNUM         *bn_exp;
     BIGNUM         *bn_mod;
 
-    if (!rsa)
+    if (!rsa || buflen == 0)
         return VAL_BAD_ARGUMENT;
 
     cp = buf;
 
-#if 1
-    if ((u_int8_t) (buf[index]) == (u_int8_t) 0) {
+    if (buf[index] == 0) {
+
+        if (buflen < 3)
+            return VAL_BAD_ARGUMENT;
+
         index += 1;
         cp = (buf + index);
         VAL_GET16(exp_len, cp);
         index += 2;
     } else {
-#endif
-        exp_len += (u_int8_t) (buf[index]);
+        exp_len += buf[index];
         index += 1;
-#if 1
     }
-#endif
 
     if (exp_len > buflen - index) {
         return VAL_BAD_ARGUMENT;
@@ -172,6 +181,10 @@ rsamd5_parse_public_key(const unsigned char *buf, int buflen, RSA * rsa)
 
     index += exp_len;
 
+    if (buflen <= index) {
+        return VAL_BAD_ARGUMENT;
+    }
+
     /*
      * Extract the modulus 
      */
@@ -192,12 +205,12 @@ rsamd5_parse_public_key(const unsigned char *buf, int buflen, RSA * rsa)
  *   of the public key modulus)."
  */
 u_int16_t
-rsamd5_keytag(const unsigned char *pubkey, int pubkey_len)
+rsamd5_keytag(const u_char *pubkey, size_t pubkey_len)
 {
     RSA            *rsa = NULL;
     BIGNUM         *modulus;
     u_int16_t       keytag = 0x0000;
-    unsigned char  *modulus_bin;
+    u_char  *modulus_bin;
     int             modulus_len;
 
     if ((rsa = RSA_new()) == NULL) {
@@ -212,7 +225,7 @@ rsamd5_keytag(const unsigned char *pubkey, int pubkey_len)
     modulus = rsa->n;
     modulus_len = BN_num_bytes(modulus);
     modulus_bin =
-        (unsigned char *) MALLOC(modulus_len * sizeof(unsigned char));
+        (u_char *) MALLOC(modulus_len * sizeof(u_char));
 
     BN_bn2bin(modulus, modulus_bin);
 
@@ -226,16 +239,16 @@ rsamd5_keytag(const unsigned char *pubkey, int pubkey_len)
 
 void
 rsamd5_sigverify(val_context_t * ctx,
-                 const unsigned char *data,
-                 int data_len,
+                 const u_char *data,
+                 size_t data_len,
                  const val_dnskey_rdata_t * dnskey,
                  const val_rrsig_rdata_t * rrsig,
                  val_astatus_t * key_status, val_astatus_t * sig_status)
 {
     char            buf[1028];
-    int             buflen = 1024;
+    size_t          buflen = 1024;
     RSA            *rsa = NULL;
-    unsigned char   md5_hash[MD5_DIGEST_LENGTH];
+    u_char   md5_hash[MD5_DIGEST_LENGTH];
 
     val_log(ctx, LOG_DEBUG,
             "rsamd5_sigverify(): parsing the public key...");
@@ -256,15 +269,15 @@ rsamd5_sigverify(val_context_t * ctx,
     }
 
     bzero(md5_hash, MD5_DIGEST_LENGTH);
-    MD5(data, data_len, (unsigned char *) md5_hash);
+    MD5(data, data_len, (u_char *) md5_hash);
     val_log(ctx, LOG_DEBUG, "rsamd5_sigverify(): MD5 hash = %s",
             get_hex_string(md5_hash, MD5_DIGEST_LENGTH, buf, buflen));
 
     val_log(ctx, LOG_DEBUG,
             "rsamd5_sigverify(): verifying RSA signature...");
 
-    if (RSA_verify(NID_md5, (unsigned char *) md5_hash, MD5_DIGEST_LENGTH,
-                   rrsig->signature, rrsig->signature_len, rsa)) {
+    if (RSA_verify(NID_md5, (u_char *) md5_hash, MD5_DIGEST_LENGTH,
+                   rrsig->signature, rrsig->signature_len, rsa) == 1) {
         val_log(ctx, LOG_INFO, "rsamd5_sigverify(): returned SUCCESS");
         RSA_free(rsa);
         *sig_status = VAL_AC_RRSIG_VERIFIED;
@@ -280,7 +293,7 @@ rsamd5_sigverify(val_context_t * ctx,
  * Returns VAL_NO_ERROR on success, other values on failure 
  */
 static int
-rsasha1_parse_public_key(const unsigned char *buf, int buflen, RSA * rsa)
+rsa_parse_public_key(const u_char *buf, size_t buflen, RSA * rsa)
 {
     int             index = 0;
     const u_char   *cp;
@@ -288,27 +301,37 @@ rsasha1_parse_public_key(const unsigned char *buf, int buflen, RSA * rsa)
     BIGNUM         *bn_exp;
     BIGNUM         *bn_mod;
 
-    if (!rsa)
+    if (!rsa || buflen == 0)
         return VAL_BAD_ARGUMENT;
 
     cp = buf;
 
-    if ((u_int8_t) (buf[index]) == (u_int8_t) 0) {
+    if (buf[index] == 0) {
+        if (buflen < 3)
+            return VAL_BAD_ARGUMENT;
         index += 1;
         cp = (buf + index);
         VAL_GET16(exp_len, cp);
         index += 2;
     } else {
-        exp_len += (u_int8_t) (buf[index]);
+        exp_len += buf[index];
         index += 1;
     }
 
+    if (index + exp_len > buflen) {
+        return VAL_BAD_ARGUMENT;
+    }
+    
     /*
      * Extract the exponent 
      */
     bn_exp = BN_bin2bn(buf + index, exp_len, NULL);
     index += exp_len;
 
+    if (buflen <= index) {
+        return VAL_BAD_ARGUMENT;
+    }
+
     /*
      * Extract the modulus 
      */
@@ -321,9 +344,9 @@ rsasha1_parse_public_key(const unsigned char *buf, int buflen, RSA * rsa)
 }
 
 void
-rsasha1_sigverify(val_context_t * ctx,
-                  const unsigned char *data,
-                  int data_len,
+rsasha_sigverify(val_context_t * ctx,
+                  const u_char *data,
+                  size_t data_len,
                   const val_dnskey_rdata_t * dnskey,
                   const val_rrsig_rdata_t * rrsig,
                   val_astatus_t * key_status, val_astatus_t * sig_status)
@@ -331,69 +354,103 @@ rsasha1_sigverify(val_context_t * ctx,
     char            buf[1028];
     int             buflen = 1024;
     RSA            *rsa = NULL;
-    unsigned char   sha1_hash[SHA_DIGEST_LENGTH];
+    u_char   sha_hash[SHA256_DIGEST_LENGTH];
+    size_t   hashlen = 0;
+    int nid = 0;
 
     val_log(ctx, LOG_DEBUG,
-            "rsasha1_sigverify(): parsing the public key...");
+            "rsasha_sigverify(): parsing the public key...");
     if ((rsa = RSA_new()) == NULL) {
         val_log(ctx, LOG_INFO,
-                "rsasha1_sigverify(): could not allocate rsa structure.");
+                "rsasha_sigverify(): could not allocate rsa structure.");
         *key_status = VAL_AC_INVALID_KEY;
         return;
     };
 
-    if (rsasha1_parse_public_key
-        (dnskey->public_key, dnskey->public_key_len,
+    if (rsa_parse_public_key
+        (dnskey->public_key, (size_t)dnskey->public_key_len,
          rsa) != VAL_NO_ERROR) {
         val_log(ctx, LOG_INFO,
-                "rsasha1_sigverify(): Error in parsing public key.");
+                "rsasha_sigverify(): Error in parsing public key.");
         RSA_free(rsa);
         *key_status = VAL_AC_INVALID_KEY;
         return;
     }
 
-    bzero(sha1_hash, SHA_DIGEST_LENGTH);
-    SHA1(data, data_len, (unsigned char *) sha1_hash);
-    val_log(ctx, LOG_DEBUG, "rsasha1_sigverify(): SHA-1 hash = %s",
-            get_hex_string(sha1_hash, SHA_DIGEST_LENGTH, buf, buflen));
+    memset(sha_hash, 0, sizeof(sha_hash));
+    if (rrsig->algorithm == ALG_RSASHA1
+#ifdef LIBVAL_NSEC3
+        || rrsig->algorithm == ALG_NSEC3_RSASHA1
+#endif
+       ) {
+        SHA1(data, data_len, sha_hash);
+        hashlen = SHA_DIGEST_LENGTH; 
+        nid = NID_sha1; 
+    } else if (rrsig->algorithm == ALG_RSASHA256) {
+        SHA256(data, data_len, sha_hash);
+        hashlen = SHA256_DIGEST_LENGTH; 
+        nid = NID_sha256; 
+    } else if (rrsig->algorithm == ALG_RSASHA512) {
+        SHA512(data, data_len, sha_hash);
+        hashlen = SHA512_DIGEST_LENGTH; 
+        nid = NID_sha512; 
+    } else {
+        val_log(ctx, LOG_INFO,
+                "rsasha_sigverify(): Unkown algorithm.");
+        RSA_free(rsa);
+        *key_status = VAL_AC_INVALID_KEY;
+        return;
+    } 
 
+    val_log(ctx, LOG_DEBUG, "rsasha_sigverify(): SHA hash = %s",
+            get_hex_string(sha_hash, hashlen, buf, buflen));
     val_log(ctx, LOG_DEBUG,
-            "rsasha1_sigverify(): verifying RSA signature...");
+            "rsasha_sigverify(): verifying RSA signature...");
 
     if (RSA_verify
-        (NID_sha1, (unsigned char *) sha1_hash, SHA_DIGEST_LENGTH,
-         rrsig->signature, rrsig->signature_len, rsa)) {
-        val_log(ctx, LOG_INFO, "rsasha1_sigverify(): returned SUCCESS");
+        (nid, sha_hash, hashlen,
+         rrsig->signature, rrsig->signature_len, rsa) == 1) {
+        val_log(ctx, LOG_INFO, "rsasha_sigverify(): returned SUCCESS");
         RSA_free(rsa);
         *sig_status = VAL_AC_RRSIG_VERIFIED;
     } else {
-        val_log(ctx, LOG_INFO, "rsasha1_sigverify(): returned FAILURE");
+        val_log(ctx, LOG_INFO, "rsasha_sigverify(): returned FAILURE");
         RSA_free(rsa);
         *sig_status = VAL_AC_RRSIG_VERIFY_FAILED;
     }
     return;
 }
 
+
 int
-ds_sha_hash_is_equal(u_int8_t * name_n,
-                     u_int8_t * rrdata,
-                     u_int16_t rrdatalen, 
-                     u_int8_t * ds_hash,
-                     u_int32_t ds_hash_len)
+ds_sha_hash_is_equal(u_char * name_n,
+                     u_char * rrdata,
+                     size_t rrdatalen, 
+                     u_char * ds_hash,
+                     size_t ds_hash_len)
 {
-    u_int8_t        ds_digest[SHA_DIGEST_LENGTH];
-    int             namelen;
+    u_char        ds_digest[SHA_DIGEST_LENGTH];
+    size_t        namelen;
     SHA_CTX         c;
+    size_t          l_index;
+    u_char        *qc_name_n;
 
     if (rrdata == NULL || ds_hash_len != SHA_DIGEST_LENGTH)
         return 0;
 
     namelen = wire_name_length(name_n);
+    qc_name_n = (u_char *) MALLOC(namelen * sizeof(u_char));
+    if (qc_name_n == NULL) {
+        return 0;
+    }
+    memcpy(qc_name_n, name_n, namelen);
+    l_index = 0;
+    lower_name(qc_name_n, &l_index);
 
     memset(ds_digest, SHA_DIGEST_LENGTH, 0);
 
     SHA1_Init(&c);
-    SHA1_Update(&c, name_n, namelen);
+    SHA1_Update(&c, qc_name_n, namelen);
     SHA1_Update(&c, rrdata, rrdatalen);
     SHA1_Final(ds_digest, &c);
 
@@ -405,25 +462,34 @@ ds_sha_hash_is_equal(u_int8_t * name_n,
 
 #ifdef HAVE_SHA_256
 int
-ds_sha256_hash_is_equal(u_int8_t * name_n,
-                        u_int8_t * rrdata,
-                        u_int16_t rrdatalen, 
-                        u_int8_t * ds_hash,
-                        u_int32_t ds_hash_len)
+ds_sha256_hash_is_equal(u_char * name_n,
+                        u_char * rrdata,
+                        size_t rrdatalen, 
+                        u_char * ds_hash,
+                        size_t ds_hash_len)
 {
-    u_int8_t        ds_digest[SHA256_DIGEST_LENGTH];
-    int             namelen;
-    SHA256_CTX         c;
+    u_char        ds_digest[SHA256_DIGEST_LENGTH];
+    size_t        namelen;
+    SHA256_CTX    c;
+    size_t          l_index;
+    u_char        *qc_name_n;
 
     if (rrdata == NULL || ds_hash_len != SHA256_DIGEST_LENGTH)
         return 0;
 
     namelen = wire_name_length(name_n);
+    qc_name_n = (u_char *) MALLOC(namelen * sizeof(u_char));
+    if (qc_name_n == NULL) {
+        return 0;
+    }
+    memcpy(qc_name_n, name_n, namelen);
+    l_index = 0;
+    lower_name(qc_name_n, &l_index);
 
     memset(ds_digest, SHA256_DIGEST_LENGTH, 0);
 
     SHA256_Init(&c);
-    SHA256_Update(&c, name_n, namelen);
+    SHA256_Update(&c, qc_name_n, namelen);
     SHA256_Update(&c, rrdata, rrdatalen);
     SHA256_Final(ds_digest, &c);
 
@@ -435,18 +501,27 @@ ds_sha256_hash_is_equal(u_int8_t * name_n,
 #endif
 
 #ifdef LIBVAL_NSEC3
-u_int8_t       *
-nsec3_sha_hash_compute(u_int8_t * qc_name_n, u_int8_t * salt,
-                       u_int8_t saltlen, u_int16_t iter, u_int8_t ** hash,
-                       u_int8_t * hashlen)
+u_char       *
+nsec3_sha_hash_compute(u_char * name_n, u_char * salt,
+                       size_t saltlen, size_t iter, u_char ** hash,
+                       size_t * hashlen)
 {
     /*
      * Assume that the caller has already performed all sanity checks 
      */
     SHA_CTX         c;
-    int             i;
+    size_t          i;
+    size_t          l_index;
+    int len = wire_name_length(name_n);
+    u_char *qc_name_n = (u_char *) MALLOC(len * sizeof(u_char));
+    if (qc_name_n == NULL) {
+        return NULL;
+    }
+    memcpy(qc_name_n, name_n, len);
+    l_index = 0;
+    lower_name(qc_name_n, &l_index);
 
-    *hash = (u_int8_t *) MALLOC(SHA_DIGEST_LENGTH * sizeof(u_int8_t));
+    *hash = (u_char *) MALLOC(SHA_DIGEST_LENGTH * sizeof(u_char));
     if (*hash == NULL)
         return NULL;
     *hashlen = SHA_DIGEST_LENGTH;
@@ -475,8 +550,8 @@ nsec3_sha_hash_compute(u_int8_t * qc_name_n, u_int8_t * salt,
 #endif
 
 char           *
-get_base64_string(unsigned char *message, int message_len, char *buf,
-                  int bufsize)
+get_base64_string(u_char *message, size_t message_len, char *buf,
+                  size_t bufsize)
 {
     BIO            *b64 = BIO_new(BIO_f_base64());
     BIO            *mem = BIO_new_mem_buf(message, message_len);
@@ -490,7 +565,7 @@ get_base64_string(unsigned char *message, int message_len, char *buf,
 }
 
 int
-decode_base64_key(char *keyptr, u_char * public_key, int keysize)
+decode_base64_key(char *keyptr, u_char * public_key, size_t keysize)
 {
     BIO            *b64;
     BIO            *mem;
