@@ -39,6 +39,9 @@ extern int      h_errno;
 static struct hostent g_hentry;
 static char     g_auxbuf[AUX_BUFLEN];
 
+extern int address_to_reverse_domain(const char *saddr, int family,
+                                  char *dadd, int dlen);
+
 /*
  * Function: bufalloc
  *
@@ -301,10 +304,10 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
     int             addr_count = 0;
     int             addr_index = 0;
     int             orig_offset = 0;
-    char            dname[NS_MAXDNAME];
     struct val_result_chain *res;
     int validated = 1;
     int trusted = 1;
+    struct val_rrset_rec *rrset;
 
     /*
      * Check parameter sanity 
@@ -323,30 +326,28 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
      * Count the number of aliases and addresses in the result 
      */
     for (res = results; res != NULL; res = res->val_rc_next) {
-        struct val_rrset_rec *rrset;
-
-        if (res->val_rc_rrset == NULL)
-            continue;
-
         rrset = res->val_rc_rrset;
 
+        if (res->val_rc_alias && rrset) {
+            val_log(ctx, LOG_DEBUG,
+                    "get_hostent_from_response(): type of record = CNAME");
+            alias_count++;
+            continue;
+        }
+
         // Get a count of aliases and addresses
         if (rrset) {
             struct val_rr_rec  *rr = rrset->val_rrset_data;
 
             while (rr) {
 
-                if (rrset->val_rrset_type_h == ns_t_cname) {
-                    val_log(ctx, LOG_DEBUG,
-                            "get_hostent_from_response(): type of record = CNAME");
-                    alias_count++;
-                } else if ((af == AF_INET)
-                           && (rrset->val_rrset_type_h == ns_t_a)) {
+                if ((af == AF_INET)
+                           && (rrset->val_rrset_type == ns_t_a)) {
                     val_log(ctx, LOG_DEBUG,
                             "get_hostent_from_response(): type of record = A");
                     addr_count++;
                 } else if ((af == AF_INET6)
-                           && (rrset->val_rrset_type_h == ns_t_aaaa)) {
+                           && (rrset->val_rrset_type == ns_t_aaaa)) {
                     val_log(ctx, LOG_DEBUG,
                             "get_hostent_from_response(): type of record = AAAA");
                     addr_count++;
@@ -361,8 +362,7 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
         (char **) bufalloc(buf, buflen, offset,
                            (alias_count + 1) * sizeof(char *));
     if (ret->h_aliases == NULL) {
-        *offset = orig_offset;
-        return NULL;
+        goto err;
     }
     ret->h_aliases[alias_count] = NULL;
 
@@ -370,8 +370,7 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
         (char **) bufalloc(buf, buflen, offset,
                            (addr_count + 1) * sizeof(char *));
     if (ret->h_addr_list == NULL) {
-        *offset = orig_offset;
-        return NULL;
+        goto err;
     }
     ret->h_addr_list[addr_count] = NULL;
 
@@ -380,7 +379,7 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
     if (results == NULL) {
         *val_status = VAL_UNTRUSTED_ANSWER;
         *h_errnop = HOST_NOT_FOUND; 
-        return NULL;
+        goto err;
     } 
    
     /*
@@ -388,123 +387,94 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
      */
     for (res = results; res != NULL; res = res->val_rc_next) {
 
+        char *alias_target = NULL;
+        rrset = res->val_rc_rrset;
+
         if (!(validated && val_isvalidated(res->val_rc_status))) 
             validated = 0;
         if (!(trusted && val_istrusted(res->val_rc_status)))
             trusted = 0;
 
-        *val_status = res->val_rc_status;
-
         /* save the non-existence state */
         if (val_does_not_exist(res->val_rc_status)) {
+            *val_status = res->val_rc_status;
             if (res->val_rc_status == VAL_NONEXISTENT_NAME ||
                 res->val_rc_status == VAL_NONEXISTENT_NAME_NOCHAIN) {
 
                 *h_errnop = HOST_NOT_FOUND;
-                break;
             } else { 
                 *h_errnop = NO_DATA;
-                break;
             }
+            break;
         }
 
-        if (res->val_rc_rrset) {
-
-            struct val_rrset_rec *rrset = res->val_rc_rrset;
-            struct val_rr_rec  *rr = rrset->val_rrset_data;
-
-            while (rr) {
-                // Handle CNAME RRs
-                if (rrset->val_rrset_type_h == ns_t_cname) {
-
-                    bzero(dname, sizeof(dname));
-                    if (ns_name_ntop
-                        (rrset->val_rrset_name_n, dname,
-                         sizeof(dname)) < 0) {
-                        *offset = orig_offset;
-                        return NULL;
-                    }
-
-                    if (alias_index >= 0) {
-                        ret->h_aliases[alias_index] =
-                            (char *) bufalloc(buf, buflen, offset,
-                                              (strlen(dname) +
-                                               1) * sizeof(char));
-                        if (ret->h_aliases[alias_index] == NULL) {
-                            *offset = orig_offset;
-                            return NULL;
-                        }
-                        memcpy(ret->h_aliases[alias_index], dname,
-                               strlen(dname) + 1);
-                        alias_index--;
-                    }
-
-                    if (!ret->h_name) {
-                        bzero(dname, sizeof(dname));
-                        if (ns_name_ntop
-                            (rr->rr_rdata, dname, sizeof(dname)) < 0) {
-                            *offset = orig_offset;
-                            return NULL;
-                        }
-                        ret->h_name =
-                            (char *) bufalloc(buf, buflen, offset,
-                                              (strlen(dname) +
-                                               1) * sizeof(char));
-                        if (ret->h_name == NULL) {
-                            *offset = orig_offset;
-                            return NULL;
-                        }
-                        memcpy(ret->h_name, dname, strlen(dname) + 1);
-                    }
+        if (res->val_rc_alias && rrset) {
+            // Handle CNAME RRs
+            if (alias_index >= 0) {
+                ret->h_aliases[alias_index] =
+                    (char *) bufalloc(buf, buflen, offset,
+                                      (strlen(rrset->val_rrset_name) + 1) * sizeof(char));
+                if (ret->h_aliases[alias_index] == NULL) {
+                    goto err;
                 }
-                // Handle A and AAAA RRs
-                else if (((af == AF_INET)
-                          && (rrset->val_rrset_type_h == ns_t_a))
-                         || ((af == AF_INET6)
-                             && (rrset->val_rrset_type_h == ns_t_aaaa))) {
-
-                    bzero(dname, sizeof(dname));
-                    if (ns_name_ntop
-                        (rrset->val_rrset_name_n, dname,
-                         sizeof(dname)) < 0) {
-                        *offset = orig_offset;
-                        return NULL;
-                    }
-
-                    if (!ret->h_name) {
-                        ret->h_name =
-                            (char *) bufalloc(buf, buflen, offset,
-                                              (strlen(dname) +
-                                               1) * sizeof(char));
-                        if (ret->h_name == NULL) {
-                            *offset = orig_offset;
-                            return NULL;
-                        }
-                        memcpy(ret->h_name, dname, strlen(dname) + 1);
-                    }
-
-                    if (strcasecmp(ret->h_name, dname) == 0) {
-                        ret->h_length = rr->rr_rdata_length_h;
-                        ret->h_addrtype = af;
-
-                        ret->h_addr_list[addr_index] =
-                            (char *) bufalloc(buf, buflen, offset,
-                                              rr->rr_rdata_length_h *
-                                              sizeof(char));
-                        if (ret->h_addr_list[addr_index] == NULL) {
-                            *offset = orig_offset;
-                            return NULL;
-                        }
-
-                        memcpy(ret->h_addr_list[addr_index], rr->rr_rdata,
-                               rr->rr_rdata_length_h);
-                        addr_index++;
-                    }
-                }
-
-                rr = rr->rr_next;
+                memcpy(ret->h_aliases[alias_index], rrset->val_rrset_name,
+                       strlen(rrset->val_rrset_name) + 1);
+                alias_index--;
             }
 
+            /* save the alias target for later use */
+            alias_target = res->val_rc_alias; 
+        } else if (rrset) {
+
+            if (((af == AF_INET)
+                      && (rrset->val_rrset_type == ns_t_a))
+                || ((af == AF_INET6)
+                      && (rrset->val_rrset_type == ns_t_aaaa))) {
+
+                struct val_rr_rec  *rr = rrset->val_rrset_data;
+
+                if (!ret->h_name) {
+                    ret->h_name =
+                            (char *) bufalloc(buf, buflen, offset,
+                                              (strlen(rrset->val_rrset_name) +
+                                               1) * sizeof(char));
+                    if (ret->h_name == NULL) {
+                        goto err;
+                    }
+                    memcpy(ret->h_name, rrset->val_rrset_name, strlen(rrset->val_rrset_name) + 1);
+                }
+
+                while (rr) {
+                    // Handle A and AAAA RRs
+                    ret->h_length = rr->rr_rdata_length;
+                    ret->h_addrtype = af;
+                    ret->h_addr_list[addr_index] =
+                            (char *) bufalloc(buf, buflen, offset,
+                                              rr->rr_rdata_length *
+                                              sizeof(char));
+                    if (ret->h_addr_list[addr_index] == NULL) {
+                        goto err;
+                    }
+
+                    memcpy(ret->h_addr_list[addr_index], rr->rr_rdata,
+                               rr->rr_rdata_length);
+                    addr_index++;
+
+                    rr = rr->rr_next;
+                }
+            }
+
+        } else {
+            /* pick up official name from the alias target */
+            if (!ret->h_name) {
+                ret->h_name = (char *) bufalloc(buf, buflen, offset,
+                                              (strlen(rrset->val_rrset_name) +
+                                               1) * sizeof(char));
+                if (ret->h_name == NULL) {
+                    goto err;
+                }
+                memcpy(ret->h_name, rrset->val_rrset_name, strlen(rrset->val_rrset_name) + 1);
+            }
         }
     }
 
@@ -516,20 +486,19 @@ get_hostent_from_response(val_context_t * ctx, int af, struct hostent *ret,
             *val_status = VAL_TRUSTED_ANSWER;
         else 
             *val_status = VAL_UNTRUSTED_ANSWER; 
-
-    } else {
-        if (*h_errnop == 0)   {
-            /* missing a proof of non-existence */
-            *val_status = VAL_UNTRUSTED_ANSWER;
-            *h_errnop = NO_DATA;
-        }
-        if (alias_count == 0) {
-            *offset = orig_offset;
-            return NULL;
-        }
+    } else if (alias_count == 0) {
+        goto err;
+    } else if (*h_errnop == 0)   {
+        /* missing a proof of non-existence for alias */
+        *val_status = VAL_UNTRUSTED_ANSWER;
+        *h_errnop = NO_DATA;
     } 
-
     return ret;
+
+err:
+    *offset = orig_offset;
+    return NULL;
+    
 }                               /* get_hostent_from_response() */
 
 
@@ -579,15 +548,19 @@ val_gethostbyname2_r(val_context_t * context,
     struct in_addr  ip4_addr;
     struct in6_addr ip6_addr;
     int             offset = 0;
-    val_status_t local_ans_status = VAL_LOCAL_ANSWER;
+    val_status_t local_ans_status = VAL_OOB_ANSWER;
     int trusted = 0;
     int             retval;
     struct val_result_chain *results = NULL;
-    u_char          name_n[NS_MAXCDNAME];
     u_int16_t       type;
-
     val_context_t *ctx = NULL;
+
+    *val_status = VAL_DONT_KNOW;
     
+    if (!name || !ret || !h_errnop || !val_status || !result || !buf) {
+        goto err;
+    }
+
     if (context == NULL) {
         if (VAL_NO_ERROR != val_create_context(NULL, &ctx)) {
             goto err; 
@@ -601,11 +574,7 @@ val_gethostbyname2_r(val_context_t * context,
             local_ans_status = VAL_TRUSTED_ANSWER;
         }
     }
-    if (!name || !ret || !h_errnop || !val_status || !result || !buf) {
-        goto err;
-    }
 
-    *val_status = VAL_DONT_KNOW;
     bzero(&ip4_addr, sizeof(struct in_addr));
     bzero(&ip6_addr, sizeof(struct in6_addr));
 
@@ -626,7 +595,7 @@ val_gethostbyname2_r(val_context_t * context,
         ret->h_aliases =
             (char **) bufalloc(buf, buflen, &offset, sizeof(char *));
         if (ret->h_aliases == NULL) {
-            goto err;      // xxx-audit: what about *offset = orig_offset; ?
+            goto err;  
         }
         ret->h_aliases[0] = 0;
 
@@ -636,18 +605,18 @@ val_gethostbyname2_r(val_context_t * context,
         ret->h_addr_list =
             (char **) bufalloc(buf, buflen, &offset, 2 * sizeof(char *));
         if (ret->h_addr_list == NULL) {
-            goto err;      // xxx-audit: what about *offset = orig_offset; ?
+            goto err;      
         }
         ret->h_addr_list[0] =
             (char *) bufalloc(buf, buflen, &offset,
                               sizeof(struct in_addr));
         if (ret->h_addr_list[0] == NULL) {
-            goto err;      // xxx-audit: what about *offset = orig_offset; ?
+            goto err;      
         }
         memcpy(ret->h_addr_list[0], &ip4_addr, sizeof(struct in_addr));
         ret->h_addr_list[1] = 0;
 
-        *val_status = local_ans_status;
+        *val_status = VAL_TRUSTED_ANSWER;
         *h_errnop = NETDB_SUCCESS;
         *result = ret;
     }
@@ -670,7 +639,7 @@ val_gethostbyname2_r(val_context_t * context,
         ret->h_aliases =
             (char **) bufalloc(buf, buflen, &offset, sizeof(char *));
         if (ret->h_aliases == NULL) {
-            goto err;      // xxx-audit: what about *offset = orig_offset; ?
+            goto err;     
         }
         ret->h_aliases[0] = 0;
 
@@ -680,18 +649,18 @@ val_gethostbyname2_r(val_context_t * context,
         ret->h_addr_list =
             (char **) bufalloc(buf, buflen, &offset, 2 * sizeof(char *));
         if (ret->h_addr_list == NULL) {
-            goto err;      // xxx-audit: what about *offset = orig_offset; ?
+            goto err;    
         }
         ret->h_addr_list[0] =
             (char *) bufalloc(buf, buflen, &offset,
                               sizeof(struct in6_addr));
         if (ret->h_addr_list[0] == NULL) {
-            goto err;      // xxx-audit: what about *offset = orig_offset; ?
+            goto err;   
         }
         memcpy(ret->h_addr_list[0], &ip6_addr, sizeof(struct in6_addr));
         ret->h_addr_list[1] = 0;
 
-        *val_status = local_ans_status;
+        *val_status = VAL_TRUSTED_ANSWER;
         *h_errnop = NETDB_SUCCESS;
         *result = ret;
 
@@ -707,20 +676,20 @@ val_gethostbyname2_r(val_context_t * context,
 
     } else {
 
-        type = ns_t_a;
         if (af == AF_INET6) {
             type = ns_t_aaaa;
+        } else {
+            type = ns_t_a;
         }
 
         /*
          * Query the validator 
          */
-        if (((retval = ns_name_pton(name, name_n, sizeof(name_n))) != -1)
-            && (VAL_NO_ERROR ==
+        if (VAL_NO_ERROR ==
                 (retval =
-                 val_resolve_and_check(ctx, name_n, ns_c_in, type,
+                 val_resolve_and_check(ctx, name, ns_c_in, type,
                                        VAL_QUERY_NO_AC_DETAIL,
-                                       &results)))) {
+                                       &results))) {
 
             /*
              * Convert the validator result into hostent 
@@ -868,3 +837,290 @@ val_gethostbyname_r(val_context_t * ctx,
     return val_gethostbyname2_r(ctx, name, AF_INET, ret, buf, buflen,
                                 result, h_errnop, val_status);
 }                               /* val_gethostbyname_r() */
+
+/*
+ * A thread-safe, re-entrant version of val_gethostbyaddr 
+ */
+int
+val_gethostbyaddr_r(val_context_t * context,
+                    const char *addr,
+                    int len,
+                    int type,
+                    struct hostent *ret,
+                    char *buf,
+                    int buflen,
+                    struct hostent **result,
+                    int *h_errnop, val_status_t * val_status)
+{
+
+    const int       addr_size = 100;
+    char            domain_string[addr_size];
+    int             ret_status = 0, bufused = 0;
+    struct val_answer_chain *val_res = NULL;
+    struct val_answer_chain *res;
+    int retval;
+    val_context_t *ctx = NULL;
+    
+    /*
+     * check misc parameters exist 
+     */
+    if (!addr || !ret || !buf || (buflen <= 0) ||
+        !result || !h_errnop || !val_status) {
+        if (h_errnop)
+            *h_errnop = NO_RECOVERY;
+        return (NO_RECOVERY);
+    }
+
+    if (context == NULL) {
+        if (VAL_NO_ERROR != (retval = val_create_context(NULL, &ctx))) {
+            *h_errnop = NO_RECOVERY;
+            return (NO_RECOVERY);
+        } 
+    } else {
+        ctx = context;
+    }
+    
+    /*
+     * default the input parameters 
+     */
+    *result = NULL;
+    ret->h_name = NULL;
+    ret->h_aliases = NULL;
+    ret->h_addr_list = NULL;
+    *h_errnop = 0;
+    *val_status = VAL_UNTRUSTED_ANSWER;
+
+    /*
+     * get the address values, only support IPv4 and IPv6 
+     */
+    if (AF_INET == type && len >= sizeof(struct in_addr)) {
+        ret->h_addrtype = type;
+        ret->h_length = sizeof(struct in_addr);
+    } else if (AF_INET6 == type && len >= sizeof(struct in6_addr)) {
+        ret->h_addrtype = type;
+        ret->h_length = sizeof(struct in6_addr);
+    } else {
+        *h_errnop = NO_RECOVERY;
+        return (NO_RECOVERY);
+    }
+
+    memset(domain_string, 0, sizeof(char) * addr_size);
+
+    if (0 !=
+        (ret_status = address_to_reverse_domain(addr, type,
+                                                domain_string, addr_size))
+        ) {
+        *h_errnop = ret_status;
+        return ret_status;
+    }
+
+    /*
+     * if there is memory, add the address to hostent's address list 
+     */
+    if ((buflen > bufused) && 
+        ((buflen - bufused) >= (ret->h_length + (sizeof(char *) * 2)))) {
+        ret->h_addr_list = (char **) (buf + bufused);
+        bufused = bufused + (sizeof(char *) * 2);
+        ret->h_addr_list[0] = buf + bufused;
+        ret->h_addr_list[1] = NULL;
+        bufused = bufused + ret->h_length;
+        memcpy(ret->h_addr_list[0], addr, ret->h_length);
+    } else {                    /* no memory, fail */
+        *h_errnop = NO_RECOVERY;
+        return (NO_RECOVERY);
+    }
+
+    if (VAL_NO_ERROR != (retval = val_get_rrset(ctx,   /* val_context_t *  */
+                                                domain_string, /* domain name */ 
+                                                ns_c_in,   /* const u_int16_t q_class */
+                                                ns_t_ptr,  /* const u_int16_t type */
+                                                0,
+                                                &val_res))) { /* struct val_answer_chain **results */
+
+        val_log(ctx, LOG_ERR, 
+                "val_gethostbyaddr_r(): val_get_rrset failed - %s", p_val_err(retval));
+        *h_errnop = NO_RECOVERY;
+        return NO_RECOVERY;
+    }
+
+    if (!val_res) {
+        *h_errnop = NO_RECOVERY;
+        return NO_RECOVERY;
+    }
+
+    for (res = val_res; res; res=res->val_ans_next) {
+
+        struct rr_rec  *rr = res->val_ans;
+        if (rr) {
+            struct rr_rec  *rr_it = NULL;
+            int             count = 0;
+            int aliases_sz = 0;
+            /*
+             * if the buffer has enough room add the first host address 
+             */
+            if (rr->rr_length < (buflen - bufused - 1)) {
+                /*
+                 * setup hostent 
+                 */
+                ret->h_name = buf + bufused;
+                ns_name_ntop(rr->rr_data, ret->h_name,
+                             (buflen - bufused));
+                bufused = bufused + strlen(ret->h_name) + 1;
+
+                rr_it = rr->rr_next;
+                /*
+                 * are there other hostnames? 
+                 */
+                if (rr_it) {
+                    /*
+                     * calculate the amount of memory we need for aliases. 
+                     */
+                    do {
+                        count++;
+                        aliases_sz = aliases_sz + rr_it->rr_length + 1;
+                    } while (NULL != (rr_it = rr_it->rr_next));
+
+                    /*
+                     * check that we have the space in the buffer 
+                     */
+                    if (buflen >=
+                        (bufused + (sizeof(char *) * (count + 1)) +
+                        aliases_sz)) {
+                        /*
+                         * assign the string pointer array 
+                         */
+                        ret->h_aliases = (char **) (buf + bufused);
+                        bufused = bufused + (sizeof(char *) * (count + 1));
+
+                        /*
+                         * assign the strings 
+                         */
+                        rr_it = rr->rr_next;
+                        count = 0;
+                        do {
+                            ret->h_aliases[count] = buf + bufused;
+                            ns_name_ntop(rr_it->rr_data,
+                                         ret->h_aliases[count],
+                                        (buflen - bufused));
+                            bufused =
+                                bufused + strlen(ret->h_aliases[count]) + 1;
+                            count++;
+                        } while (NULL != (rr_it = rr_it->rr_next));
+                        /*
+                         * mark end of array 
+                         */
+                        ret->h_aliases[count] = NULL;
+                    }
+                    /*
+                     * else we didn't have enough memory for the aliases.  They
+                     * will be ignored with only one hostname returned 
+                     */
+                }                   /* else there are no other hostnames/aliases */
+
+            } else {                /* else there is not enough room for even one host name, fail */
+                ret->h_name = NULL;
+                *h_errnop = NO_RECOVERY;
+                return NO_RECOVERY;
+            }
+            break;
+
+        } else if  (val_does_not_exist(res->val_ans_status)) {
+                    
+            if ((res->val_ans_status == VAL_NONEXISTENT_TYPE) ||
+                (res->val_ans_status == VAL_NONEXISTENT_TYPE_NOCHAIN)) {
+                    *h_errnop = NO_DATA;
+            } else if ((res->val_ans_status == VAL_NONEXISTENT_NAME) ||
+                       (res->val_ans_status == VAL_NONEXISTENT_NAME_NOCHAIN)) {
+                    *h_errnop = HOST_NOT_FOUND;
+            }
+
+            *result = ret;
+            return *h_errnop;
+        }
+    }
+
+    if (!res) { /* no rrset, but a succesful return from the query?, fail */
+        ret->h_name = NULL;
+        *h_errnop = NO_RECOVERY;
+        return NO_RECOVERY;
+    }
+
+    /* set the value of merged trusted and validated status values */
+    if (val_isvalidated(res->val_ans_status))
+        *val_status = VAL_VALIDATED_ANSWER; 
+    else if (val_istrusted(res->val_ans_status))
+        *val_status = VAL_TRUSTED_ANSWER; 
+
+    /*
+     * no error, set result 
+     */
+    *result = ret;
+    return *h_errnop;
+
+}                               /* val_getthostbyaddr_r */
+
+
+/*
+ * A old version of gethostbyaddr for use with validator
+ */
+struct hostent *
+val_gethostbyaddr(val_context_t * context,
+                  const char *addr,
+                  int len, int type, val_status_t * val_status)
+{
+    /*
+     * static buffer size for hostent is set to 512 
+     */
+    const int       buflen = 512;
+    static char     buffer[512];        /* compiler doesn't consider a const, constant */
+    static struct hostent ret_hostent;
+
+    struct hostent *result_hostent = NULL;
+    int             errnum = 0;
+    val_context_t *ctx = NULL;
+    
+    if (context == NULL) {
+        if (VAL_NO_ERROR != val_create_context(NULL, &ctx)) {
+            h_errno = NO_RECOVERY;
+            return NULL;
+        } 
+    } else {
+        ctx = context;
+    }
+
+    /*
+     * set defaults for static values 
+     */
+    memset(buffer, 0, sizeof(char) * buflen);
+    ret_hostent.h_name = NULL;
+    ret_hostent.h_aliases = NULL;
+    ret_hostent.h_addrtype = 0;
+    ret_hostent.h_length = 0;
+    ret_hostent.h_addr_list = NULL;
+
+    int             response = val_gethostbyaddr_r(ctx,
+                                                   addr, len, type,
+                                                   &ret_hostent,
+                                                   buffer, buflen,
+                                                   &result_hostent,
+                                                   &errnum,
+                                                   val_status);
+
+    if (response != 0) {
+        h_errno = response;
+        return NULL;
+    }
+    /*
+     * should have succeeded, if memory doesn't match, fail. 
+     */
+    else if (&ret_hostent != result_hostent) {
+        h_errno = NO_RECOVERY;
+        return NULL;
+    }
+
+    /*
+     * success 
+     */
+    h_errno = 0;
+    return &ret_hostent;
+}                               /* val_gethostbyaddr */
