@@ -46,7 +46,7 @@
      */\
     if (VAL_NO_ERROR != (retval =\
          val_get_token(buf_ptr, end_ptr, line_number, token, sizeof(token), endst,\
-                       CONF_COMMENT, CONF_END_STMT)))\
+                       CONF_COMMENT, CONF_END_STMT, 1)))\
         return retval;\
 } while (0)
 
@@ -300,11 +300,6 @@ parse_trust_anchor(char **buf_ptr, char *end_ptr, policy_entry_t * pol_entry,
     ta_pol->publickey = NULL;
     ta_pol->ds = NULL;
 
-    /*
-     * XXX We may want to have another token that specifies if 
-     * XXX this is a DS or a DNSKEY
-     * XXX Assume public key for now
-     */
     pkstr = &ta_token[0];
     endptr = pkstr + strlen(ta_token);
     
@@ -334,7 +329,7 @@ parse_trust_anchor(char **buf_ptr, char *end_ptr, policy_entry_t * pol_entry,
                
        /* 
         *  Treat as though we have a DNSKEY, even if neither 
-        *  DNSKEY nor DS is specified (backwards compatibility
+        *  DNSKEY nor DS is specified (backwards compatibility)
         */ 
         if (VAL_NO_ERROR !=
             (retval =
@@ -476,8 +471,6 @@ parse_zone_security_expectation(char **buf_ptr, char *end_ptr, policy_entry_t *
 
     if (!strcmp(se_token, ZONE_SE_IGNORE_MSG))
         zone_status = ZONE_SE_IGNORE;
-    else if (!strcmp(se_token, ZONE_SE_TRUSTED_MSG))
-        zone_status = ZONE_SE_TRUSTED;
     else if (!strcmp(se_token, ZONE_SE_DO_VAL_MSG))
         zone_status = ZONE_SE_DO_VAL;
     else if (!strcmp(se_token, ZONE_SE_UNTRUSTED_MSG))
@@ -564,8 +557,8 @@ parse_dlv_trust_points(char **buf_ptr, char *end_ptr, policy_entry_t * pol_entry
     char            dlv_token[TOKEN_MAX];
     struct dlv_policy *dlv_pol;
     int             retval;
-    int             len;
-    u_int8_t        zone_n[NS_MAXCDNAME];
+    size_t        len;
+    u_char        zone_n[NS_MAXCDNAME];
 
     if ((buf_ptr == NULL) || (*buf_ptr == NULL) || (end_ptr == NULL) || 
         (pol_entry == NULL) || (line_number == NULL))
@@ -584,7 +577,7 @@ parse_dlv_trust_points(char **buf_ptr, char *end_ptr, policy_entry_t * pol_entry
         return VAL_OUT_OF_MEMORY;
     }
     len = wire_name_length(zone_n);
-    dlv_pol->trust_point = (u_int8_t *) MALLOC (len * sizeof(u_int8_t));
+    dlv_pol->trust_point = (u_char *) MALLOC (len * sizeof(u_char));
     if (dlv_pol->trust_point == NULL) {
         FREE(dlv_pol);
         return VAL_OUT_OF_MEMORY;
@@ -600,7 +593,7 @@ int
 free_dlv_trust_points(policy_entry_t * pol_entry)
 {
     if (pol_entry && pol_entry->pol) {
-        u_int8_t *tp = ((struct dlv_policy *)(pol_entry->pol))->trust_point;
+        u_char *tp = ((struct dlv_policy *)(pol_entry->pol))->trust_point;
         if (tp)
             FREE(tp);
         FREE(pol_entry->pol);
@@ -637,8 +630,11 @@ val_get_token(char **buf_ptr,
               char *end_ptr,
               int *line_number,
               char *conf_token,
-              int conf_limit, int *endst, 
-              const char *comment_c, char endstmt_c)
+              int conf_limit, 
+              int *endst, 
+              const char *comment_c, 
+              char endstmt_c,
+              int ignore_space)
 {
     int             i = 0;
     int             quoted = 0;
@@ -683,8 +679,12 @@ val_get_token(char **buf_ptr,
 
     i = 0;
 
-    while (*buf_ptr < end_ptr && 
-           (!isspace((c = **buf_ptr)) || quoted || escaped)) {
+    while (*buf_ptr < end_ptr) {
+        c = **buf_ptr;
+        /* if spaces are meant to be delimiters, break */
+        if (!ignore_space && isspace(c) && !quoted && !escaped) {
+            break;
+        }
            
         (*buf_ptr)++;
         if (i == conf_limit)
@@ -704,7 +704,7 @@ val_get_token(char **buf_ptr,
                 break;
 
             case '\n' :
-                if (!escaped) {
+                if (!escaped && !quoted) {
                     goto done;
                 }
                 (*line_number)++;
@@ -812,7 +812,7 @@ parse_local_answer_gopt(char **buf_ptr, char *end_ptr, int *line_number,
     if (VAL_NO_ERROR != (retval = 
         val_get_token(buf_ptr, end_ptr, line_number, 
                       token, sizeof(token), endst,
-                      CONF_COMMENT, CONF_END_STMT))) {
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
         return retval;
     }
     if ((endst && (strlen(token) == 0)) ||
@@ -820,9 +820,9 @@ parse_local_answer_gopt(char **buf_ptr, char *end_ptr, int *line_number,
         return VAL_CONF_PARSE_ERROR;
     }
 
-    if (!strcmp(token, TRUST_LOCAL_GOPT_YES_STR))
+    if (!strcmp(token, TRUST_OOB_GOPT_YES_STR))
         g_opt->local_is_trusted = 1;
-    else if (!strcmp(token, TRUST_LOCAL_GOPT_NO_STR))
+    else if (!strcmp(token, TRUST_OOB_GOPT_NO_STR))
         g_opt->local_is_trusted = 0;
     else
         return VAL_CONF_PARSE_ERROR;
@@ -844,7 +844,7 @@ parse_edns0_size_gopt(char **buf_ptr, char *end_ptr, int *line_number,
     if (VAL_NO_ERROR != (retval = 
         val_get_token(buf_ptr, end_ptr, line_number, 
                       token, sizeof(token), endst,
-                      CONF_COMMENT, CONF_END_STMT))) {
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
         return retval;
     }
     if ((endst && (strlen(token) == 0)) ||
@@ -872,7 +872,7 @@ parse_enable_disable_gopt(int *type, char **buf_ptr, char *end_ptr, int *line_nu
     if (VAL_NO_ERROR != (retval = 
         val_get_token(buf_ptr, end_ptr, line_number, 
                       token, sizeof(token), endst,
-                      CONF_COMMENT, CONF_END_STMT))) {
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
         return retval;
     }
     if ((endst && (strlen(token) == 0)) ||
@@ -907,7 +907,7 @@ parse_log_target_gopt(char **buf_ptr, char *end_ptr, int *line_number,
     if (VAL_NO_ERROR != (retval = 
         val_get_token(buf_ptr, end_ptr, line_number, 
                       token, sizeof(token), endst,
-                      CONF_COMMENT, CONF_END_STMT))) {
+                      CONF_COMMENT, CONF_END_STMT, 0))) {
         return retval;
     }
     if ((endst && (strlen(token) == 0)) ||
@@ -950,7 +950,7 @@ get_global_options(char **buf_ptr, char *end_ptr,
         if (VAL_NO_ERROR != (retval = 
             val_get_token(buf_ptr, end_ptr, line_number, 
                           token, sizeof(token), &endst,
-                          CONF_COMMENT, CONF_END_STMT))) {
+                          CONF_COMMENT, CONF_END_STMT, 0))) {
             goto err;
         }
         if (endst && (strlen(token) == 0)) {
@@ -962,7 +962,8 @@ get_global_options(char **buf_ptr, char *end_ptr,
         }
 
         /* parse the option value based on the type */
-        if (!strcmp(token, GOPT_TRUST_LOCAL_STR)) {
+        if (!strcmp(token, GOPT_TRUST_OOB_STR) ||
+            !strcmp(token, GOPT_TRUST_LOCAL_STR)) {
             if (VAL_NO_ERROR != 
                     (retval = parse_local_answer_gopt(buf_ptr, end_ptr,
                                                       line_number, &endst, *g_opt))) {
@@ -1043,7 +1044,7 @@ get_next_policy_fragment(char **buf_ptr, char *end_ptr, char *scope,
         if (VAL_NO_ERROR !=
             (retval =
              val_get_token(buf_ptr, end_ptr, line_number, token, sizeof(token), &endst,
-                       CONF_COMMENT, CONF_END_STMT)))
+                       CONF_COMMENT, CONF_END_STMT, 0)))
             return retval;
         if (*buf_ptr >= end_ptr)
             return VAL_NO_ERROR;
@@ -1081,7 +1082,7 @@ get_next_policy_fragment(char **buf_ptr, char *end_ptr, char *scope,
         if (VAL_NO_ERROR !=
             (retval =
              val_get_token(buf_ptr, end_ptr, line_number, token, sizeof(token), &endst,
-                       CONF_COMMENT, CONF_END_STMT))) {
+                       CONF_COMMENT, CONF_END_STMT, 0))) {
             FREE(label);
             return retval;
         }
@@ -1108,7 +1109,7 @@ get_next_policy_fragment(char **buf_ptr, char *end_ptr, char *scope,
              */
             if (VAL_NO_ERROR != (retval = 
                 val_get_token(buf_ptr, end_ptr, line_number, token, sizeof(token), &endst,
-                       CONF_COMMENT, CONF_END_STMT))) {
+                       CONF_COMMENT, CONF_END_STMT, 0))) {
 
                 free_policy_entry(pol, index);
                 pol = NULL;
@@ -1525,7 +1526,7 @@ read_next_val_config_file(val_context_t *ctx,
                 if (VAL_NO_ERROR != (retval = 
                         val_get_token(&buf_ptr, end_ptr, &line_number, 
                                       token, sizeof(token), &endst,
-                                      CONF_COMMENT, CONF_END_STMT))) {
+                                      CONF_COMMENT, CONF_END_STMT, 0))) {
                     val_log(ctx, LOG_ERR, 
                             "read_next_val_config_file(): Error in line %d of %s ",
                             line_number, dnsval_c->dnsval_conf);
@@ -1823,10 +1824,8 @@ read_val_config_file(val_context_t * ctx, char *scope, int *is_override)
      * Re-initialize caches 
      */
     free_query_chain(ctx->q_list);
-    free_authentication_chain(ctx->a_list);
 
     ctx->q_list = NULL;
-    ctx->a_list = NULL;
     ctx->dnsval_l = dnsval_list;
 
     CTX_UNLOCK_VALPOL(ctx);
@@ -1861,6 +1860,9 @@ destroy_respol(val_context_t * ctx)
 static int
 parse_name_server(char *cp, struct name_server **ns)
 { 
+    short port_num = DNS_PORT;
+    char *cpt, addr[IPADDR_STRING_MAX];
+
     struct sockaddr_storage serv_addr;
     struct sockaddr_in *sin = (struct sockaddr_in *)&serv_addr;
 #ifdef VAL_IPV6
@@ -1901,11 +1903,31 @@ parse_name_server(char *cp, struct name_server **ns)
     (*ns)->ns_next = NULL;
     (*ns)->ns_number_of_addresses = 0;
 
+    /*
+     * Look for port number in address string
+     * syntax of '[address]:port'
+     */
+    cpt = cp;
+    if ( (*cpt == '[') && (cpt = strchr(cpt,']')) ) {
+      if ( IPADDR_STRING_MAX < (cpt - cp) ) {
+	goto parse_err;
+      }
+      bzero(addr, IPADDR_STRING_MAX);
+      strncpy(addr, (cp + 1), (cpt - cp - 1));
+      cp = addr;
+      if ( (*(++cpt) = ':') && (0 == (port_num = atoi(++cpt))) ) {
+	  goto parse_err;
+      }
+    }
+
+    /*
+     * convert address string
+     */
     bzero(&serv_addr, sizeof(serv_addr));
     if (inet_pton(AF_INET, cp, &address.v4) > 0) {
         sin->sin_family = AF_INET;     // host byte order
         sin->sin_addr = address.v4;
-        sin->sin_port = htons(DNS_PORT);       // short, network byte order
+        sin->sin_port = htons(port_num);       // short, network byte order
     }
     else {
 #ifdef VAL_IPV6
@@ -1914,7 +1936,7 @@ parse_name_server(char *cp, struct name_server **ns)
 
         sin6->sin6_family = AF_INET6;     // host byte order
         memcpy(&sin6->sin6_addr, &address.v6, sizeof(address.v6));
-        sin6->sin6_port = htons(DNS_PORT);       // short, network byte order
+        sin6->sin6_port = htons(port_num);       // short, network byte order
 #else
         goto parse_err;
 #endif
@@ -1952,7 +1974,7 @@ read_res_config_file(val_context_t * ctx)
     struct name_server *ns_head = NULL;
     struct name_server *ns_tail = NULL;
     struct name_server *ns = NULL;
-    u_int8_t zone_n[NS_MAXCDNAME];
+    u_char zone_n[NS_MAXCDNAME];
     struct stat sb;
     char *buf_ptr, *end_ptr;
     char *buf = NULL;
@@ -2021,7 +2043,7 @@ read_res_config_file(val_context_t * ctx)
         if (VAL_NO_ERROR !=
             (retval =
              val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                       ALL_COMMENTS, ZONE_END_STMT))) {
+                       ALL_COMMENTS, ZONE_END_STMT, 0))) {
             goto err;
         }
 
@@ -2039,12 +2061,17 @@ read_res_config_file(val_context_t * ctx)
             if (VAL_NO_ERROR !=
                 (retval =
                 val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                           ALL_COMMENTS, ZONE_END_STMT))) {
+                           ALL_COMMENTS, ZONE_END_STMT, 0))) {
+                val_log(ctx, LOG_WARNING,
+			"read_res_config_file(): error getting nameserver token!");
                 goto err;
             }
             ns = NULL;
-            if (VAL_NO_ERROR != parse_name_server(token, &ns))
+            if (VAL_NO_ERROR != parse_name_server(token, &ns)) {
+                val_log(ctx, LOG_WARNING,
+			"read_res_config_file(): error parsing nameserver token!");
                 goto err;
+	    }
             if (ns != NULL) {
                 if (ns_tail == NULL) {
                     ns_head = ns;
@@ -2065,7 +2092,7 @@ read_res_config_file(val_context_t * ctx)
             if (VAL_NO_ERROR !=
                 (retval =
                 val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                           ALL_COMMENTS, ZONE_END_STMT))) {
+                           ALL_COMMENTS, ZONE_END_STMT, 0))) {
                 goto err;
             }
             ns = NULL;
@@ -2075,7 +2102,7 @@ read_res_config_file(val_context_t * ctx)
             if (VAL_NO_ERROR !=
                 (retval =
                 val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                           ALL_COMMENTS, ZONE_END_STMT))) {
+                           ALL_COMMENTS, ZONE_END_STMT, 0))) {
                 goto err;
             }
             if (ns != NULL) {
@@ -2093,7 +2120,7 @@ read_res_config_file(val_context_t * ctx)
             if (VAL_NO_ERROR !=
                 (retval =
                 val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                           ALL_COMMENTS, ZONE_END_STMT))) {
+                           ALL_COMMENTS, ZONE_END_STMT, 0))) {
                 goto err;
             }
             if (ctx->search)
@@ -2230,7 +2257,7 @@ read_root_hints_file(val_context_t * ctx)
         if (VAL_NO_ERROR !=
             (retval =
             val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                   ZONE_COMMENT, ZONE_END_STMT))) {
+                   ZONE_COMMENT, ZONE_END_STMT, 0))) {
             goto err;
         }
         if (buf_ptr >= end_ptr) {
@@ -2253,7 +2280,7 @@ read_root_hints_file(val_context_t * ctx)
         if (VAL_NO_ERROR !=
             (retval =
              val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                       ZONE_COMMENT, ZONE_END_STMT))) {
+                       ZONE_COMMENT, ZONE_END_STMT, 0))) {
             goto err;
             
         }
@@ -2272,7 +2299,7 @@ read_root_hints_file(val_context_t * ctx)
         if (VAL_NO_ERROR !=
             (retval =
              val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                       ZONE_COMMENT, ZONE_END_STMT))) {
+                       ZONE_COMMENT, ZONE_END_STMT, 0))) {
             goto err;
         }
         class_h = res_nametoclass(token, &success);
@@ -2295,7 +2322,7 @@ read_root_hints_file(val_context_t * ctx)
             if (VAL_NO_ERROR !=
                 (retval =
                 val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                       ZONE_COMMENT, ZONE_END_STMT))) {
+                       ZONE_COMMENT, ZONE_END_STMT, 0))) {
                 goto err;
             }
         }
@@ -2316,7 +2343,7 @@ read_root_hints_file(val_context_t * ctx)
         if (VAL_NO_ERROR !=
             (retval =
              val_get_token(&buf_ptr, end_ptr, &line_number, token, sizeof(token), &endst,
-                       ZONE_COMMENT, ZONE_END_STMT))) {
+                       ZONE_COMMENT, ZONE_END_STMT, 0))) {
             goto err;
         }
         if (type_h == ns_t_a) {
@@ -2550,9 +2577,9 @@ parse_etc_hosts(const char *name)
 
 
 int 
-val_add_valpolicy(val_context_t *context, const char *keyword, 
-                  char *zone, char *value, long ttl, 
-                  val_policy_entry_t **pol)
+val_add_valpolicy(val_context_t *context, 
+                  void *policy_definition,
+                  val_policy_handle_t **pol)
 {
     int index;
     u_char zone_n[NS_MAXCDNAME];
@@ -2566,7 +2593,16 @@ val_add_valpolicy(val_context_t *context, const char *keyword,
     val_context_t *ctx = NULL;
     int retval;
 
-    if (keyword == NULL || zone == NULL || value == NULL || pol == NULL)
+    libval_policy_definition_t *libval_pol;
+
+    if (policy_definition == NULL || pol == NULL)
+        return VAL_BAD_ARGUMENT;
+
+    libval_pol = (libval_policy_definition_t *) policy_definition;
+
+    if (libval_pol->keyword == NULL || 
+        libval_pol->zone == NULL || 
+        libval_pol->value == NULL)
         return VAL_BAD_ARGUMENT;
 
     if (context == NULL) {
@@ -2580,7 +2616,7 @@ val_add_valpolicy(val_context_t *context, const char *keyword,
     
     /* find the policy according to the keyword */
     for (index = 0; index < MAX_POL_TOKEN; index++) {
-        if (!strcmp(keyword, conf_elem_array[index].keyword)) {
+        if (!strcmp(libval_pol->keyword, conf_elem_array[index].keyword)) {
             break;
         }
     }
@@ -2588,18 +2624,18 @@ val_add_valpolicy(val_context_t *context, const char *keyword,
         return VAL_BAD_ARGUMENT;
     }
 
-    if (ns_name_pton(zone, zone_n, NS_MAXCDNAME) == -1) {
+    if (ns_name_pton(libval_pol->zone, zone_n, NS_MAXCDNAME) == -1) {
         return VAL_BAD_ARGUMENT;
     } 
 
-    if (ttl > 0) {
+    if (libval_pol->ttl > 0) {
         gettimeofday(&tv, NULL);
-        ttl_x = ttl + tv.tv_sec;
+        ttl_x = libval_pol->ttl + tv.tv_sec;
     } else
         ttl_x = -1;
         
-    buf_ptr = value;
-    end_ptr = value+strlen(value);
+    buf_ptr = libval_pol->value;
+    end_ptr = libval_pol->value+strlen(libval_pol->value);
 
     pol_entry = (policy_entry_t *) MALLOC (sizeof(policy_entry_t));
     if (pol_entry == NULL) {
@@ -2628,13 +2664,13 @@ val_add_valpolicy(val_context_t *context, const char *keyword,
         LOCK_QC_EX(q);
         /* Should never fail when holding above locks */
         if (NULL != namename(q->qc_name_n, zone_n)) {
-            zap_query(ctx, q);
+            free_query_chain_structure(q);
             if (pol_entry->exp_ttl > 0)
                 q->qc_ttl_x = pol_entry->exp_ttl;
         }
         UNLOCK_QC(q);    
     }
-    *pol = (val_policy_entry_t *) MALLOC (sizeof(val_policy_entry_t));
+    *pol = (val_policy_handle_t *) MALLOC (sizeof(val_policy_handle_t));
     if (*pol == NULL) {
         retval = VAL_OUT_OF_MEMORY;
         goto err;
@@ -2654,7 +2690,7 @@ err:
 }  
 
 int 
-val_remove_valpolicy(val_context_t *context, val_policy_entry_t *pol)
+val_remove_valpolicy(val_context_t *context, val_policy_handle_t *pol)
 {
     val_context_t *ctx = NULL;
     policy_entry_t *p, *prev;
@@ -2701,7 +2737,7 @@ val_remove_valpolicy(val_context_t *context, val_policy_entry_t *pol)
         LOCK_QC_EX(q);
         /* Should never fail when holding above locks */
         if (NULL != namename(q->qc_name_n, p->zone_n)) {
-            zap_query(ctx, q);
+            free_query_chain_structure(q);
         }
         UNLOCK_QC(q);    
     }
