@@ -44,7 +44,8 @@
 		ref->answers = NULL; \
 		ref->qnames = NULL; \
 		ref->pending_glue_ns = NULL; \
-		ref->merged_glue_ns = NULL; \
+		ref->cur_pending_glue_ns = NULL; \
+		ref->saved_zonecut_n = NULL; \
 		ref->learned_zones = NULL; \
 } while(0)
 
@@ -64,6 +65,7 @@
             }\
             FREE(ns_address);\
             ns_address = NULL;\
+            break; \
         }\
     }\
 }while(0)
@@ -76,71 +78,73 @@ void            my_free(void *p, char *filename, int lineno);
 void           *my_malloc(size_t t, char *filename, int lineno);
 char           *my_strdup(const char *str, char *filename, int lineno);
 
-int             label_bytes_cmp(const u_int8_t * field1, int length1,
-                                const u_int8_t * field2, int length2);
-int             labelcmp(const u_int8_t * name1, const u_int8_t * name2, int label_cnt);
-int             namecmp(const u_int8_t * name1, const u_int8_t * name2);
-u_int8_t *      namename(u_int8_t * big_name, u_int8_t * little_name);
+int             label_bytes_cmp(const u_char * field1, size_t length1,
+                                const u_char * field2, size_t length2);
+int             labelcmp(const u_char * name1, const u_char * name2, 
+                        size_t label_cnt);
+int             namecmp(const u_char * name1, const u_char * name2);
+u_char *      namename(u_char * big_name, u_char * little_name);
 #ifdef LIBVAL_NSEC3
-void            base32hex_encode(u_int8_t * in, u_int8_t inlen,
-                                 u_int8_t ** out, u_int8_t * outlen);
+void            base32hex_encode(u_char * in, size_t inlen,
+                                 u_char ** out, size_t * outlen);
 #endif
-u_int16_t       wire_name_labels(const u_int8_t * field);
-u_int16_t       wire_name_length(const u_int8_t * field);
+size_t          wire_name_labels(const u_char * field);
+size_t          wire_name_length(const u_char * field);
 
 void            res_sq_free_rr_recs(struct val_rr_rec **rr);
 void            res_sq_free_rrset_recs(struct rrset_rec **set);
 int             add_to_qname_chain(struct qname_chain **qnames,
-                                   const u_int8_t * name_n);
+                                   const u_char * name_n);
 int             qname_chain_first_name(struct qname_chain *qnames,
-                                       const u_int8_t * name_n);
+                                       const u_char * name_n);
 void            free_qname_chain(struct qname_chain **qnames);
 void            free_domain_info_ptrs(struct domain_info *di);
-int             is_tail(u_int8_t * full, u_int8_t * tail);
-int             nxt_sig_match(u_int8_t * owner, u_int8_t * next,
-                              u_int8_t * signer);
-int             add_to_set(struct rrset_rec *rr_set, u_int16_t rdata_len_h,
-                           u_int8_t * rdata);
-int             add_as_sig(struct rrset_rec *rr_set, u_int16_t rdata_len_h,
-                           u_int8_t * rdata);
-int             init_rr_set(struct rrset_rec *new_set, u_int8_t * name_n,
+int             is_tail(u_char * full, u_char * tail);
+int             nxt_sig_match(u_char * owner, u_char * next,
+                              u_char * signer);
+int             add_to_set(struct rrset_rec *rr_set, size_t rdata_len_h,
+                           u_char * rdata);
+int             add_as_sig(struct rrset_rec *rr_set, size_t rdata_len_h,
+                           u_char * rdata);
+int             init_rr_set(struct rrset_rec *new_set, u_char * name_n,
                             u_int16_t type_h, u_int16_t set_type_h,
                             u_int16_t class_h, u_int32_t ttl_h,
-                            u_int8_t * hptr, int from_section,
+                            u_char * hptr, int from_section,
                             int authoritive_answer,
                             struct name_server *respondent_server);
 
 struct rrset_rec *find_rr_set(struct name_server *respondent_server,
                               struct rrset_rec **the_list,
-                              u_int8_t * name_n,
+                              u_char * name_n,
                               u_int16_t type_h,
                               u_int16_t set_type_h,
                               u_int16_t class_h,
                               u_int32_t ttl_h,
-                              u_int8_t * hptr,
-                              u_int8_t * rdata_n,
+                              u_char * hptr,
+                              u_char * rdata_n,
                               int from_section,
                               int authoritive_answer,
-                              u_int8_t * zonecut_n);
+                              u_char * zonecut_n);
 
-int             decompress(u_int8_t ** rdata,
-                           u_int8_t * response,
-                           int rdata_index,
-                           u_int8_t * end,
-                           u_int16_t type_h, u_int16_t * rdata_len_h);
+int             decompress(u_char ** rdata,
+                           u_char * response,
+                           size_t rdata_index,
+                           u_char * end,
+                           u_int16_t type_h, 
+                           size_t * rdata_len_h);
 
-int             extract_from_rr(u_int8_t * response,
-                                int *response_index,
-                                u_int8_t * end,
-                                u_int8_t * name_n,
+int             extract_from_rr(u_char * response,
+                                size_t *response_index,
+                                u_char * end,
+                                u_char * name_n,
                                 u_int16_t * type_h,
                                 u_int16_t * set_type_h,
                                 u_int16_t * class_h,
                                 u_int32_t * ttl_h,
-                                u_int16_t * rdata_length_h,
-                                int *rdata_index);
-void            lower_name(u_int8_t rdata[], size_t * index);
-void            lower(u_int16_t type_h, u_int8_t * rdata, int len);
+                                size_t * rdata_length_h,
+                                size_t *rdata_index);
+void            lower_name(u_char rdata[], size_t * index);
+void            lower(u_int16_t type_h, u_char * rdata, size_t len);
 struct val_rr_rec  *copy_rr_rec(u_int16_t type_h, struct val_rr_rec *r,
                             int dolower);
 struct val_rr_rec  *copy_rr_rec_list(u_int16_t type_h, struct val_rr_rec *o_rr,
@@ -148,8 +152,8 @@ struct val_rr_rec  *copy_rr_rec_list(u_int16_t type_h, struct val_rr_rec *o_rr,
 int             link_rr(struct val_rr_rec **cs, struct val_rr_rec *cr);
 struct rrset_rec *copy_rrset_rec(struct rrset_rec *rr_set);
 struct rrset_rec *copy_rrset_rec_list(struct rrset_rec *rr_set);
-int             register_query(struct query_list **q, u_int8_t * name_n,
-                               u_int32_t type_h, u_int8_t * zone_n);
+int             register_query(struct query_list **q, u_char * name_n,
+                               u_int16_t type_h, u_char * zone_n);
 void            deregister_queries(struct query_list **q);
 void            merge_rrset_recs(struct rrset_rec **dest,
                                  struct rrset_rec *new_info);
