@@ -35,13 +35,13 @@
  */
 static void
 get_clock_skew(val_context_t *ctx,
-               u_int8_t *name_n,
+               u_char *name_n,
                int *skew,
                u_int32_t *ttl_x)
 {
     policy_entry_t *cs_pol, *cs_cur;
-    u_int8_t       *p;
-    int             name_len;
+    u_char       *p;
+    size_t       name_len;
 
     if (ctx == NULL || name_n == NULL || skew == NULL || ttl_x == NULL) {
         val_log(ctx, LOG_DEBUG, "get_clock_skew(): Cannot check for clock skew policy, bad args"); 
@@ -64,7 +64,7 @@ get_clock_skew(val_context_t *ctx,
          */
         for (; cs_cur; cs_cur = cs_cur->next) {
             int             root_zone = 0;
-            if (!namecmp(cs_cur->zone_n, (const u_int8_t *) ""))
+            if (!namecmp(cs_cur->zone_n, (const u_char *) ""))
                 root_zone = 1;
             else {
                 /*
@@ -95,11 +95,11 @@ get_clock_skew(val_context_t *ctx,
 /*
  * Verify a signature, given the data and the dnskey 
  */
-static void
+static int 
 val_sigverify(val_context_t * ctx,
               int is_a_wildcard,
-              const unsigned char *data,
-              int data_len,
+              const u_char *data,
+              size_t data_len,
               const val_dnskey_rdata_t * dnskey,
               const val_rrsig_rdata_t * rrsig,
               val_astatus_t * dnskey_status, val_astatus_t * sig_status,
@@ -116,7 +116,7 @@ val_sigverify(val_context_t * ctx,
     if ((dnskey->flags & ZONE_KEY_FLAG) == 0) {
         val_log(ctx, LOG_INFO, "val_sigverify(): DNSKEY with tag=%d is not a zone key", dnskey->key_tag);
         *dnskey_status = VAL_AC_INVALID_KEY;
-        return;
+        return 0;
     }
 
     /*
@@ -127,7 +127,7 @@ val_sigverify(val_context_t * ctx,
                 "val_sigverify(): Invalid protocol field in DNSKEY with tag=%d: %d",
                 dnskey->protocol, dnskey->key_tag);
         *dnskey_status = VAL_AC_UNKNOWN_DNSKEY_PROTOCOL;
-        return;
+        return 0;
     }
 
     /*
@@ -138,7 +138,7 @@ val_sigverify(val_context_t * ctx,
                 "val_sigverify(): Algorithm mismatch between DNSKEY (%d) and RRSIG (%d) records.",
                 dnskey->algorithm, rrsig->algorithm);
         *sig_status = VAL_AC_RRSIG_ALGORITHM_MISMATCH;
-        return;
+        return 0;
     }
 
 
@@ -177,7 +177,7 @@ val_sigverify(val_context_t * ctx,
                         "val_sigverify(): Signature not yet valid. Current time (%s) is less than signature inception time (%s).",
                         currTime, incpTime);
                 *sig_status = VAL_AC_RRSIG_NOTYETACTIVE;
-                return;
+                return 0;
             } else {
                 val_log(ctx, LOG_DEBUG,
                         "val_sigverify(): Signature not yet valid, but within acceptable skew.");
@@ -214,7 +214,7 @@ val_sigverify(val_context_t * ctx,
                         "val_sigverify(): Signature expired. Current time (%s) is greater than signature expiration time (%s).",
                         currTime, exprTime);
                 *sig_status = VAL_AC_RRSIG_EXPIRED;
-                return;
+                return 0;
             } else {
                 val_log(ctx, LOG_DEBUG,
                         "val_sigverify(): Signature expired, but within acceptable skew.");
@@ -228,8 +228,8 @@ val_sigverify(val_context_t * ctx,
     switch (rrsig->algorithm) {
 
     case ALG_RSAMD5:
-        rsamd5_sigverify(ctx, data, data_len, dnskey, rrsig, dnskey_status,
-                         sig_status);
+        rsamd5_sigverify(ctx, data, data_len, dnskey, rrsig, 
+                         dnskey_status, sig_status);
         break;
 
 #ifdef LIBVAL_NSEC3
@@ -244,7 +244,9 @@ val_sigverify(val_context_t * ctx,
     case ALG_NSEC3_RSASHA1:
 #endif
     case ALG_RSASHA1:
-        rsasha1_sigverify(ctx, data, data_len, dnskey, rrsig,
+    case ALG_RSASHA256:
+    case ALG_RSASHA512:
+        rsasha_sigverify(ctx, data, data_len, dnskey, rrsig,
                           dnskey_status, sig_status);
         break;
 
@@ -267,7 +269,10 @@ val_sigverify(val_context_t * ctx,
             if (clock_skew > 0)
                 *sig_status = VAL_AC_RRSIG_VERIFIED_SKEW;
         }
+        return 1;
     }
+
+    return 0;
 }
 
 /*
@@ -281,24 +286,24 @@ val_sigverify(val_context_t * ctx,
  */
 static int
 predict_sigbuflength(struct rrset_rec *rr_set,
-                     size_t * field_length, int *signer_length)
+                     size_t * field_length, size_t *signer_length)
 {
     struct val_rr_rec  *rr;
     int             owner_length;
 
     /** Input has already been NULL-checked **/
-    owner_length = wire_name_length(rr_set->rrs.val_rrset_name_n);
+    owner_length = wire_name_length(rr_set->rrs_name_n);
 
     *signer_length =
-        wire_name_length(&rr_set->rrs.val_rrset_sig->rr_rdata[SIGNBY]);
+        wire_name_length(&rr_set->rrs_sig->rr_rdata[SIGNBY]);
 
     if (*signer_length == 0)
         return VAL_BAD_ARGUMENT;
 
     *field_length = SIGNBY + (*signer_length);
 
-    for (rr = rr_set->rrs.val_rrset_data; rr; rr = rr->rr_next)
-        *field_length += owner_length + ENVELOPE + rr->rr_rdata_length_h;
+    for (rr = rr_set->rrs_data; rr; rr = rr->rr_next)
+        *field_length += owner_length + ENVELOPE + rr->rr_rdata_length;
 
     return VAL_NO_ERROR;
 }
@@ -307,34 +312,34 @@ predict_sigbuflength(struct rrset_rec *rr_set,
  * Create the buffer over which the signature is to be verified
  */
 static int
-make_sigfield(u_int8_t ** field,
+make_sigfield(u_char ** field,
               size_t * field_length,
               struct rrset_rec *rr_set,
               struct val_rr_rec *rr_sig, int is_a_wildcard)
 {
     struct val_rr_rec  *curr_rr;
-    int             index;
-    int             signer_length;
-    int             owner_length;
+    size_t          index;
+    size_t          signer_length;
+    size_t          owner_length;
     u_int16_t       type_n;
     u_int16_t       class_n;
     u_int32_t       ttl_n;
     u_int16_t       rdata_length_n;
-    u_int8_t        lowered_owner_n[NS_MAXCDNAME];
+    u_char          lowered_owner_n[NS_MAXCDNAME];
     size_t          l_index;
     int             retval;
 
     if ((field == NULL) || (field_length == NULL) || (rr_set == NULL) ||
-        (rr_sig == NULL) || (rr_set->rrs.val_rrset_name_n == NULL) ||
-        (rr_set->rrs.val_rrset_sig == NULL) || 
-        (rr_set->rrs.val_rrset_sig->rr_rdata == NULL))
+        (rr_sig == NULL) || (rr_set->rrs_name_n == NULL) ||
+        (rr_set->rrs_sig == NULL) || 
+        (rr_set->rrs_sig->rr_rdata == NULL))
         return VAL_BAD_ARGUMENT;
 
     if ((retval = predict_sigbuflength(rr_set, field_length, &signer_length)) !=
         VAL_NO_ERROR)
         return retval;
 
-    *field = (u_int8_t *) MALLOC(*field_length);
+    *field = (u_char *) MALLOC(*field_length * sizeof(u_char));
 
     if (*field == NULL)
         return VAL_OUT_OF_MEMORY;
@@ -344,24 +349,24 @@ make_sigfield(u_int8_t ** field,
      */
 
     memcpy(&ttl_n, &rr_sig->rr_rdata[TTL], sizeof(u_int32_t));
-    rr_set->rrs.val_rrset_ttl_h = ntohl(ttl_n);
+    rr_set->rrs_ttl_h = ntohl(ttl_n);
 
     /*
      * While we're at it, we'll gather other common info, specifically
      * network ordered numbers (type, class) and name length.
      */
 
-    owner_length = wire_name_length(rr_set->rrs.val_rrset_name_n);
+    owner_length = wire_name_length(rr_set->rrs_name_n);
 
     if (owner_length == 0)
         goto err;
 
-    memcpy(lowered_owner_n, rr_set->rrs.val_rrset_name_n, owner_length);
+    memcpy(lowered_owner_n, rr_set->rrs_name_n, owner_length);
     l_index = 0;
     lower_name(lowered_owner_n, &l_index);
 
-    type_n = htons(rr_set->rrs.val_rrset_type_h);
-    class_n = htons(rr_set->rrs.val_rrset_class_h);
+    type_n = htons(rr_set->rrs_type_h);
+    class_n = htons(rr_set->rrs_class_h);
 
     /*
      * Copy in the SIG RDATA (up to the signature 
@@ -371,13 +376,15 @@ make_sigfield(u_int8_t ** field,
     if ((index + SIGNBY + signer_length) > *field_length)
         goto err;
     memcpy(&(*field)[index], rr_sig->rr_rdata, SIGNBY + signer_length);
+    l_index = 0;
+    lower_name(&(*field)[index+SIGNBY], &l_index);
     index += SIGNBY + signer_length;
 
     /*
      * For each record of data, copy in the envelope & the lower cased rdata 
      */
 
-    for (curr_rr = rr_set->rrs.val_rrset_data; curr_rr;
+    for (curr_rr = rr_set->rrs_data; curr_rr;
          curr_rr = curr_rr->rr_next) {
         if (curr_rr->rr_rdata == NULL)
             goto err;
@@ -390,16 +397,16 @@ make_sigfield(u_int8_t ** field,
             /*
              * Construct the original name 
              */
-            u_char          wcard_n[NS_MAXCDNAME];
-            u_int8_t       *np = lowered_owner_n;
-            int             i;
-            int             outer_len;
+            u_char wcard_n[NS_MAXCDNAME];
+            u_char *np = lowered_owner_n;
+            int    i;
+            size_t outer_len;
 
             for (i = 0; i < is_a_wildcard; i++)
                 np += np[0] + 1;
             outer_len = wire_name_length(np);
 
-            wcard_n[0] = (u_int8_t) 1;
+            wcard_n[0] = (u_char) 1;
             wcard_n[1] = '*';
             if ((outer_len + 2) > sizeof(wcard_n))
                 goto err;
@@ -430,15 +437,15 @@ make_sigfield(u_int8_t ** field,
          * Now the RR-specific info, the length and the data 
          */
 
-        rdata_length_n = htons(curr_rr->rr_rdata_length_h);
-        if ((index + sizeof(u_int16_t) + curr_rr->rr_rdata_length_h)
+        rdata_length_n = htons(curr_rr->rr_rdata_length);
+        if ((index + sizeof(u_int16_t) + curr_rr->rr_rdata_length)
             > *field_length)
             goto err;
         memcpy(&(*field)[index], &rdata_length_n, sizeof(u_int16_t));
         index += sizeof(u_int16_t);
         memcpy(&(*field)[index], curr_rr->rr_rdata,
-               curr_rr->rr_rdata_length_h);
-        index += curr_rr->rr_rdata_length_h;
+               curr_rr->rr_rdata_length);
+        index += curr_rr->rr_rdata_length;
     }
 
     *field_length = index;
@@ -456,11 +463,11 @@ make_sigfield(u_int8_t ** field,
  * the rrsig
  */
 static int
-identify_key_from_sig(struct val_rr_rec *sig, u_int8_t ** name_n,
+identify_key_from_sig(struct val_rr_rec *sig, u_char ** name_n,
                       u_int16_t * footprint_n)
 {
     if ((sig == NULL) || (sig->rr_rdata == NULL) || (name_n == NULL) ||
-        (footprint_n == NULL) || (sig->rr_rdata_length_h < SIGNBY)) {
+        (footprint_n == NULL) || (sig->rr_rdata_length < SIGNBY)) {
         if (name_n != NULL)
             *name_n = NULL;
         if (footprint_n != NULL)
@@ -477,9 +484,9 @@ identify_key_from_sig(struct val_rr_rec *sig, u_int8_t ** name_n,
 /*
  * helper function for a set of verify-related operations
  */
-static void
+static int
 do_verify(val_context_t * ctx,
-          u_int8_t *zone_n,
+          u_char *zone_n,
           val_astatus_t * dnskey_status,
           val_astatus_t * sig_status,
           struct rrset_rec *the_set,
@@ -490,22 +497,23 @@ do_verify(val_context_t * ctx,
      * Use the crypto routines to verify the signature
      */
 
-    u_int8_t       *ver_field;
+    u_char       *ver_field;
     size_t          ver_length;
     int             ret_val;
     val_rrsig_rdata_t rrsig_rdata;
     int clock_skew = 0;
     u_int32_t ttl_x = 0;
+    int retval = 0;
 
     /*
      * Wildcard expansions for DNSKEYs and DSs are not permitted
      */
     if (is_a_wildcard &&
-        ((the_set->rrs.val_rrset_type_h == ns_t_ds) ||
-         (the_set->rrs.val_rrset_type_h == ns_t_dnskey))) {
+        ((the_set->rrs_type_h == ns_t_ds) ||
+         (the_set->rrs_type_h == ns_t_dnskey))) {
         val_log(ctx, LOG_INFO, "do_verify(): Invalid DNSKEY or DS record - cannot be wildcard expanded");
         *dnskey_status = VAL_AC_INVALID_KEY;
-        return;
+        return 0;
     }
 
     if ((ret_val = make_sigfield(&ver_field, &ver_length, the_set, the_sig,
@@ -517,34 +525,34 @@ do_verify(val_context_t * ctx,
                 "do_verify(): Could not construct signature field for verification: %s", 
                 p_val_err(ret_val));
         *sig_status = VAL_AC_INVALID_RRSIG;
-        return;
+        return 0;
     }
 
     /*
      * Find the signature - no memory is malloc'ed for this operation  
      */
 
-    if (-1 == val_parse_rrsig_rdata(the_sig->rr_rdata, 
-                                     the_sig->rr_rdata_length_h,
-                                     &rrsig_rdata)) {
+    if (VAL_NO_ERROR != val_parse_rrsig_rdata(the_sig->rr_rdata, 
+                                   the_sig->rr_rdata_length,
+                                   &rrsig_rdata)) {
         if (ver_field)
             FREE(ver_field);
         val_log(ctx, LOG_INFO, 
                 "do_verify(): Could not parse signature field");
         *sig_status = VAL_AC_INVALID_RRSIG;
-        return;
+        return 0;
     }
 
     rrsig_rdata.next = NULL;
 
     get_clock_skew(ctx, zone_n, &clock_skew, &ttl_x);
     /* the state is valid for only as long as the policy validity period */
-    SET_MIN_TTL(the_set->rrs.val_rrset_ttl_x, ttl_x);
+    SET_MIN_TTL(the_set->rrs_ttl_x, ttl_x);
 
     /*
      * Perform the verification 
      */
-    val_sigverify(ctx, is_a_wildcard, ver_field, ver_length, the_key,
+    retval = val_sigverify(ctx, is_a_wildcard, ver_field, ver_length, the_key,
                   &rrsig_rdata, dnskey_status, sig_status, clock_skew);
 
     if (rrsig_rdata.signature != NULL) {
@@ -553,7 +561,7 @@ do_verify(val_context_t * ctx,
     }
 
     FREE(ver_field);
-    return;
+    return retval;
 }
 
 /*
@@ -561,8 +569,8 @@ do_verify(val_context_t * ctx,
  */
 int
 ds_hash_is_equal(val_context_t *ctx,
-                 u_int8_t ds_hashtype, u_int8_t * ds_hash,
-                 u_int32_t ds_hash_len, u_int8_t * name_n,
+                 u_int8_t ds_hashtype, u_char * ds_hash,
+                 size_t ds_hash_len, u_char * name_n,
                  struct val_rr_rec *dnskey, val_astatus_t * ds_status)
 {
     if ((dnskey == NULL) || (ds_hash == NULL) || (name_n == NULL)) {
@@ -575,7 +583,7 @@ ds_hash_is_equal(val_context_t *ctx,
      */
     if (ds_hashtype == ALG_DS_HASH_SHA1) {
         return ds_sha_hash_is_equal(name_n, dnskey->rr_rdata,
-                                    dnskey->rr_rdata_length_h, 
+                                    (size_t)dnskey->rr_rdata_length, 
                                     ds_hash, ds_hash_len);
 
     } 
@@ -583,7 +591,7 @@ ds_hash_is_equal(val_context_t *ctx,
 #ifdef HAVE_SHA_256
     else if (ds_hashtype == ALG_DS_HASH_SHA256) {
         return ds_sha256_hash_is_equal(name_n, dnskey->rr_rdata,
-                                       dnskey->rr_rdata_length_h, 
+                                       (size_t)dnskey->rr_rdata_length, 
                                        ds_hash, ds_hash_len);
     } 
 #endif
@@ -599,13 +607,13 @@ static int
 check_label_count(struct rrset_rec *the_set,
                   struct val_rr_rec *the_sig, int *is_a_wildcard)
 {
-    u_int8_t        owner_labels;;
-    u_int8_t        sig_labels;
+    size_t        owner_labels;;
+    size_t        sig_labels;
 
     if ((the_set == NULL) || (the_sig == NULL) || (is_a_wildcard == NULL))
         return 0;
 
-    owner_labels = wire_name_labels(the_set->rrs.val_rrset_name_n);
+    owner_labels = wire_name_labels(the_set->rrs_name_n);
     sig_labels = the_sig->rr_rdata[RRSIGLABEL] + 1;
 
     if (sig_labels > owner_labels)
@@ -623,32 +631,35 @@ check_label_count(struct rrset_rec *the_set,
  * VAL_AC_WCARD_VERIFIED : if sigs were wildcard verified 
  * the exact error
  */
-
 #define SET_STATUS(savedstatus, rr, newstatus) \
 	do { \
 		rr->rr_status = newstatus; \
-        if (savedstatus == VAL_AC_TRUST_NOCHK ||\
-            savedstatus == VAL_AC_TRUST)\
-            ; /* do nothing */\
+        if (\
+            savedstatus == VAL_AC_TRUST_NOCHK &&\
+            newstatus == VAL_AC_DNSKEY_NOMATCH) {\
+                savedstatus = VAL_AC_NOT_VERIFIED; \
+        }\
+        else if (\
+            savedstatus == VAL_AC_TRUST_NOCHK ||\
+            savedstatus == VAL_AC_TRUST ||\
+            savedstatus == VAL_AC_VERIFIED ||\
+            savedstatus == VAL_AC_WCARD_VERIFIED ||\
+            newstatus == VAL_AC_UNSET ||\
+            newstatus == VAL_AC_UNKNOWN_ALGORITHM_LINK)\
+                ; /* do nothing */\
         /* Any success is good */\
-        else if (newstatus == VAL_AC_RRSIG_VERIFIED ||\
+        else if (\
+            newstatus == VAL_AC_RRSIG_VERIFIED ||\
             newstatus == VAL_AC_RRSIG_VERIFIED_SKEW) \
-            savedstatus = VAL_AC_VERIFIED;\
-        else if (newstatus == VAL_AC_WCARD_VERIFIED ||\
-                 newstatus == VAL_AC_WCARD_VERIFIED_SKEW)\
-            savedstatus = VAL_AC_WCARD_VERIFIED;\
+                savedstatus = VAL_AC_VERIFIED;\
+        else if (\
+            newstatus == VAL_AC_WCARD_VERIFIED ||\
+            newstatus == VAL_AC_WCARD_VERIFIED_SKEW)\
+                savedstatus = VAL_AC_WCARD_VERIFIED;\
         /* we don't already have success and what we receive is bad */ \
-        else if ((savedstatus != VAL_AC_VERIFIED) && \
-                 (savedstatus != VAL_AC_WCARD_VERIFIED) &&\
-                 (newstatus != VAL_AC_UNSET) &&\
-                 /* success values for DNSKEYS are not relevant */\
-                 (newstatus != VAL_AC_SIGNING_KEY) && \
-                 (newstatus != VAL_AC_TRUST_POINT) && \
-                 (newstatus != VAL_AC_UNKNOWN_ALGORITHM_LINK) && \
-                 (newstatus != VAL_AC_VERIFIED_LINK)){\
-            savedstatus = VAL_AC_NOT_VERIFIED; \
+        else {\
+                savedstatus = VAL_AC_NOT_VERIFIED; \
         }\
-        /* else leave savedstatus untouched */\
 	} while (0)
 
 void
@@ -658,51 +669,52 @@ verify_next_assertion(val_context_t * ctx,
 {
     struct rrset_rec *the_set;
     struct val_rr_rec  *the_sig;
-    u_int8_t       *signby_name_n;
+    u_char       *signby_name_n;
     u_int16_t       signby_footprint_n;
     val_dnskey_rdata_t dnskey;
     int             is_a_wildcard;
     struct val_rr_rec  *nextrr;
     struct val_rr_rec  *keyrr;
     u_int16_t       tag_h;
+    int             is_verified = 0;
 
-    if ((as == NULL) || (as->_as.ac_data == NULL) || (the_trust == NULL)) {
+    if ((as == NULL) || (as->val_ac_rrset.ac_data == NULL) || (the_trust == NULL)) {
         val_log(ctx, LOG_INFO, "verify_next_assertion(): Cannot verify assertion - no data");
         return;
     }
 
-    the_set = as->_as.ac_data;
+    the_set = as->val_ac_rrset.ac_data;
     dnskey.public_key = NULL;
 
-    if (the_set->rrs.val_rrset_sig == NULL) {
+    if (the_set->rrs_sig == NULL) {
         val_log(ctx, LOG_INFO, "verify_next_assertion(): RRSIG is missing");
         as->val_ac_status = VAL_AC_RRSIG_MISSING;
         return;
     }
 
-    if (the_set->rrs.val_rrset_type_h != ns_t_dnskey) {
+    if (the_set->rrs_type_h != ns_t_dnskey) {
         /*
          * trust path contains the key 
          */
-        if (the_trust->_as.ac_data == NULL) {
+        if (the_trust->val_ac_rrset.ac_data == NULL) {
             val_log(ctx, LOG_INFO, "verify_next_assertion(): Key is empty");
             as->val_ac_status = VAL_AC_DNSKEY_MISSING;
             return;
         }
-        keyrr = the_trust->_as.ac_data->rrs.val_rrset_data;
+        keyrr = the_trust->val_ac_rrset.ac_data->rrs_data;
     } else {
         /*
          * data itself contains the key 
          */
-        if (the_set->rrs.val_rrset_data == NULL) {
+        if (the_set->rrs_data == NULL) {
             val_log(ctx, LOG_INFO, "verify_next_assertion(): Key is empty");
             as->val_ac_status = VAL_AC_DNSKEY_MISSING;
             return;
         }
-        keyrr = the_set->rrs.val_rrset_data;
+        keyrr = the_set->rrs_data;
     }
 
-    for (the_sig = the_set->rrs.val_rrset_sig;
+    for (the_sig = the_set->rrs_sig;
          the_sig; the_sig = the_sig->rr_next) {
 
         /*
@@ -728,11 +740,11 @@ verify_next_assertion(val_context_t * ctx,
 
         tag_h = ntohs(signby_footprint_n);
         for (nextrr = keyrr; nextrr; nextrr = nextrr->rr_next) {
-            if (-1 == val_parse_dnskey_rdata(nextrr->rr_rdata,
-                                             nextrr->rr_rdata_length_h,
+            if (VAL_NO_ERROR != val_parse_dnskey_rdata(nextrr->rr_rdata,
+                                             nextrr->rr_rdata_length,
                                              &dnskey)) {
                 val_log(ctx, LOG_INFO, "verify_next_assertion(): Cannot parse DNSKEY data");
-                SET_STATUS(as->val_ac_status, nextrr, VAL_AC_INVALID_KEY);
+                nextrr->rr_status = VAL_AC_INVALID_KEY;
                 continue;
             }
 
@@ -745,40 +757,30 @@ verify_next_assertion(val_context_t * ctx,
                 continue;
             }
 
-            val_log(ctx, LOG_DEBUG, "verify_next_assertion(): Found matching DNSKEY for RRSIG");
+            val_log(ctx, LOG_DEBUG, "verify_next_assertion(): Found potential matching DNSKEY for RRSIG");
 
             /*
              * check the signature 
              */
-            do_verify(ctx, signby_name_n,
+            is_verified = do_verify(ctx, signby_name_n,
                       &nextrr->rr_status,
                       &the_sig->rr_status,
                       the_set, the_sig, &dnskey, is_a_wildcard);
 
-
-            if (the_sig->rr_status == VAL_AC_RRSIG_VERIFIED ||
-                the_sig->rr_status == VAL_AC_RRSIG_VERIFIED_SKEW ||
-                the_sig->rr_status == VAL_AC_WCARD_VERIFIED ||
-                the_sig->rr_status == VAL_AC_WCARD_VERIFIED_SKEW) {
-
-                SET_STATUS(as->val_ac_status, the_sig, the_sig->rr_status);
-                SET_STATUS(as->val_ac_status, nextrr, nextrr->rr_status);
-                if (nextrr->rr_status == VAL_AC_UNSET) {
-                    nextrr->rr_status = VAL_AC_SIGNING_KEY;
-                }
-                break;
-            }
-
             /*
              * There might be multiple keys with the same key tag; set this as
              * the signing key only if we dont have other status for this key
              */
             SET_STATUS(as->val_ac_status, the_sig, the_sig->rr_status);
-            SET_STATUS(as->val_ac_status, nextrr, nextrr->rr_status);
             if (nextrr->rr_status == VAL_AC_UNSET) {
                 nextrr->rr_status = VAL_AC_SIGNING_KEY;
             }
 
+            // if success, break
+            if (is_verified) {
+                break;
+            }
+
             if (dnskey.public_key != NULL) {
                 FREE(dnskey.public_key);
             }
@@ -786,23 +788,28 @@ verify_next_assertion(val_context_t * ctx,
         }
 
         if (nextrr == NULL) {
-            val_log(ctx, LOG_INFO, "verify_next_assertion(): No DNSKEY matched for this RRSIG");
+            val_log(ctx, LOG_INFO, "verify_next_assertion(): Could not link this RRSIG to a DNSKEY");
             SET_STATUS(as->val_ac_status, the_sig, VAL_AC_DNSKEY_NOMATCH);
+        } 
 
-        } else if (as->val_ac_status == VAL_AC_TRUST_NOCHK || 
-                    as->val_ac_status == VAL_AC_TRUST) {
+        if (as->val_ac_status == VAL_AC_TRUST ||
+            (is_verified && 
+                    (as->val_ac_status == VAL_AC_TRUST_NOCHK || 
+                     nextrr->rr_status == VAL_AC_TRUST_POINT))) {
+            /* we've verified a trust anchor */
 
-            if (nextrr->rr_status == VAL_AC_TRUST_POINT && 
-                    (the_sig->rr_status == VAL_AC_RRSIG_VERIFIED ||
-                     the_sig->rr_status == VAL_AC_RRSIG_VERIFIED_SKEW)) {
-                    as->val_ac_status = VAL_AC_TRUST; 
-                    val_log(ctx, LOG_INFO, "verify_next_assertion(): verification traces back to trust anchor");
+            as->val_ac_status = VAL_AC_TRUST; 
+            val_log(ctx, LOG_INFO, "verify_next_assertion(): verification traces back to trust anchor");
+            if (dnskey.public_key != NULL) {
+                FREE(dnskey.public_key);
+                dnskey.public_key = NULL;
             }
+            return;
             
-        } else if (the_set->rrs.val_rrset_type_h == ns_t_dnskey && 
-                     (the_sig->rr_status == VAL_AC_RRSIG_VERIFIED ||
-                     the_sig->rr_status == VAL_AC_RRSIG_VERIFIED_SKEW ||
-                     the_sig->rr_status == VAL_AC_ALGORITHM_NOT_SUPPORTED)) {
+        /* else check if we're trying to verify some key in the authentication chain */
+        } else if ( the_set->rrs_type_h == ns_t_dnskey && 
+                    as->val_ac_status != VAL_AC_TRUST &&
+                     (is_verified || the_sig->rr_status == VAL_AC_ALGORITHM_NOT_SUPPORTED)) {
 
             /* Check if we have reached our trust key */
             
@@ -817,30 +824,37 @@ verify_next_assertion(val_context_t * ctx,
              * follow the trust path 
              */
             struct val_rr_rec  *dsrec =
-                the_trust->_as.ac_data->rrs.val_rrset_data;
+                the_trust->val_ac_rrset.ac_data->rrs_data;
             while (dsrec) {
                 val_ds_rdata_t  ds;
-                if (-1 == val_parse_ds_rdata(dsrec->rr_rdata,
-                                   dsrec->rr_rdata_length_h, &ds)) {
-                    val_log(ctx, LOG_INFO, "verify_next_assertion(): Unknown DS algorithm");
-                    SET_STATUS(as->val_ac_status, dsrec, VAL_AC_UNKNOWN_ALGORITHM_LINK);
+                int retval = val_parse_ds_rdata(dsrec->rr_rdata,
+                                   dsrec->rr_rdata_length, &ds);
+                if(retval == VAL_NOT_IMPLEMENTED) {
+                    val_log(ctx, LOG_INFO, "verify_next_assertion(): DS hash not supported");
+                    dsrec->rr_status = VAL_AC_ALGORITHM_NOT_SUPPORTED;
+                } else if (retval != VAL_NO_ERROR) {
+                    val_log(ctx, LOG_INFO, "verify_next_assertion(): DS parse error");
+                    dsrec->rr_status = VAL_AC_INVALID_DS;
                 } else {
 
                     if (DNSKEY_MATCHES_DS(ctx, &dnskey, &ds, 
-                            the_set->rrs.val_rrset_name_n, nextrr, 
+                            the_set->rrs_name_n, nextrr, 
                             &dsrec->rr_status)) {
 
-                        if (the_sig->rr_status == VAL_AC_RRSIG_VERIFIED ||
-                            the_sig->rr_status == VAL_AC_RRSIG_VERIFIED_SKEW)
-                            SET_STATUS(as->val_ac_status, nextrr,
-                                       VAL_AC_VERIFIED_LINK);
+                        val_log(ctx, LOG_DEBUG, 
+                                "verify_next_assertion(): DNSKEY tag (%d) matches DS tag (%d)",
+                                (&dnskey)->key_tag,                                         
+                                (&ds)->d_keytag);
+                        if (is_verified)
+                            nextrr->rr_status = VAL_AC_VERIFIED_LINK;
                         else
-                            SET_STATUS(as->val_ac_status, nextrr,
-                                       VAL_AC_UNKNOWN_ALGORITHM_LINK);
+                            nextrr->rr_status = VAL_AC_UNKNOWN_ALGORITHM_LINK;
 
                         FREE(ds.d_hash);
+                        ds.d_hash = NULL;
                         if (dnskey.public_key) {
                             FREE(dnskey.public_key);
+                            dnskey.public_key = NULL;
                         }
                         /*
                         * the first match is enough 
@@ -870,8 +884,10 @@ verify_next_assertion(val_context_t * ctx,
         }
     }
 
-    if (as->val_ac_status == VAL_AC_TRUST_NOCHK) {
-        /* we were not able to verify our trust anchor */
-        as->val_ac_status = VAL_AC_NO_TRUST_ANCHOR;
+    /* If we reach here and we're a keyset, we either didn't verify the keyset or
+       didn't verify the link from the key to the DS 
+     */ 
+    if (the_set->rrs_type_h == ns_t_dnskey){
+        as->val_ac_status = VAL_AC_NO_LINK;
     }
 }
